"""Tests for Chopingredient"""

from MAST.ingredients.chopingredient import ChopIngredient

import unittest
from unittest import SkipTest
import os
import time
import MAST
import pymatgen
from MAST.utility import dirutil
from MAST.utility import MASTFile
import shutil

testname="chop_test_update"
testdir = dirutil.get_test_dir(testname)
oldscratch = os.getenv("MAST_SCRATCH")

class TestUpdateChildrenIngredient(unittest.TestCase):

    def setUp(self):
        os.chdir(testdir)
        if not os.path.isdir("writedir"):
            os.mkdir("writedir")
        if not os.path.isdir("writedir/next_ingred"):
            os.mkdir("writedir/next_ingred")
        if not os.path.isdir("writedir/single_label1"):
            os.mkdir("writedir/single_label1")
        if not os.path.isdir("writedir/neb_labelinit-labelfin"):
            os.mkdir("writedir/neb_labelinit-labelfin")
        if not os.path.isdir("writedir/single_phonon_label1"):
            os.mkdir("writedir/single_phonon_label1")
        os.environ['MAST_SCRATCH'] = testdir

    def tearDown(self):
        tearlist = list()
        tearlist.append("writedir")
        os.environ['MAST_SCRATCH'] = oldscratch
        #tearlist.append("writedir/single_label1")
        #tearlist.append("writedir/next_ingred")
        #tearlist.append("writedir/neb_labelinit-labelfin")
        #tearlist.append("writedir/single_phonon_label1")
        for foldername in tearlist:
            try:
                shutil.rmtree(foldername)
            except OSError:
                pass

    def test___init__(self):
        self.assertTrue(True)
        #self.testclass.__init__(**kwargs)

    def test__fullpath_childname(self):
        ingdir="%s/writedir/single_label1" % testdir
        topmetad = MASTFile("files/top_metadata_single")
        topmetad.data.append("origin_dir = %s/files\n" % testdir) #give origin directory
        topmetad.to_file("writedir/metadata.txt")
        metad = MASTFile("files/metadata_single")
        metad.to_file("%s/metadata.txt" % ingdir)
        kdict=dict()
        kdict['mast_program'] = 'vasp'
        my_structure = pymatgen.io.vaspio.Poscar.from_file("files/perfect_structure").structure
        myuci = ChopIngredient(name=ingdir,program_keys=kdict,structure=my_structure)
        fullpath = myuci._fullpath_childname("next_ingred")
        self.assertEqual(fullpath, "%s/writedir/next_ingred" % testdir)
        #self.testclass._fullpath_childname(childname)

    def test_give_structure(self):
        ingdir="%s/writedir/single_label1" % testdir
        recipedir="%s/writedir" % testdir
        topmetad = MASTFile("files/top_metadata_single")
        topmetad.data.append("origin_dir = %s/files\n" % testdir) #give origin directory
        topmetad.to_file("writedir/metadata.txt")
        metad = MASTFile("files/metadata_single")
        metad.to_file("%s/metadata.txt" % ingdir)
        kdict=dict()
        kdict['mast_program'] = 'vasp'
        my_structure = pymatgen.io.vaspio.Poscar.from_file("files/perfect_structure").structure
        myrelaxed = MASTFile("files/relaxed_structure")
        myrelaxed.to_file("%s/CONTCAR" % ingdir)
        myuci = ChopIngredient(name=ingdir,program_keys=kdict, structure=my_structure)
        myuci.give_structure("next_ingred")
        givenstr = MASTFile("%s/writedir/next_ingred/POSCAR" % testdir)
        self.assertEqual(myrelaxed.data, givenstr.data)
        #self.testclass.give_structure(childname)

    def test_give_neb_structures_to_neb(self):
        ingdir="%s/writedir/neb_labelinit-labelfin" % testdir
        recipedir="%s/writedir" % testdir
        topmetad = MASTFile("files/top_metadata_neb")
        topmetad.data.append("origin_dir = %s/files\n" % testdir) #give origin directory
        topmetad.to_file("writedir/metadata.txt")
        metad = MASTFile("files/metadata_neb")
        metad.to_file("%s/metadata.txt" % ingdir)
        kdict=dict()
        kdict['mast_program'] = 'vasp_neb'
        kdict['mast_neb_settings']=dict()
        kdict['mast_neb_settings']['images'] = 3
        my_structure = pymatgen.io.vaspio.Poscar.from_file("files/perfect_structure").structure
        myrelaxed=dict()
        for subdir in ['00','01','02','03','04']:
            os.mkdir("writedir/neb_labelinit-labelfin/%s" % subdir)
            myrelaxed[subdir] = MASTFile("files/POSCAR_%s" % subdir)
            myrelaxed[subdir].to_file("writedir/neb_labelinit-labelfin/%s/CONTCAR" % subdir)
        myuci = ChopIngredient(name=ingdir,program_keys=kdict, structure=my_structure)
        myuci.give_neb_structures_to_neb("next_ingred")
        for subdir in ['01','02','03']:
            givenstr = MASTFile("%s/writedir/next_ingred/parent_structure_labelinit-labelfin_%s" % (testdir,subdir))
            self.assertEqual(myrelaxed[subdir].data, givenstr.data)
        #self.testclass.give_neb_structures_to_neb(childname)

    def test_give_saddle_structure(self):
        ingdir="%s/writedir/neb_labelinit-labelfin" % testdir
        recipedir="%s/writedir" % testdir
        topmetad = MASTFile("files/top_metadata_neb")
        topmetad.data.append("origin_dir = %s/files\n" % testdir) #give origin directory
        topmetad.to_file("writedir/metadata.txt")
        metad = MASTFile("files/metadata_neb")
        metad.to_file("%s/metadata.txt" % ingdir)
        kdict=dict()
        kdict['mast_program'] = 'vasp_neb'
        kdict['images'] = 3
        my_structure = pymatgen.io.vaspio.Poscar.from_file("files/perfect_structure").structure
        myrelaxed=dict()
        myosz=dict()
        mywav=dict()
        mychg=dict()
        for subdir in ['00','01','02','03','04']:
            os.mkdir("writedir/neb_labelinit-labelfin/%s" % subdir)
            myrelaxed[subdir] = MASTFile("files/POSCAR_%s" % subdir)
            myrelaxed[subdir].to_file("writedir/neb_labelinit-labelfin/%s/CONTCAR" % subdir)
            myosz[subdir] = MASTFile("files/OSZICAR_%s" % subdir)
            myosz[subdir].to_file("writedir/neb_labelinit-labelfin/%s/OSZICAR" % subdir)
            mychg[subdir] = MASTFile("files/CHGCAR")
            mychg[subdir].to_file("writedir/neb_labelinit-labelfin/%s/CHGCAR" % subdir)
            mywav[subdir] = MASTFile("files/WAVECAR")
            mywav[subdir].to_file("writedir/neb_labelinit-labelfin/%s/WAVECAR" % subdir)
        myuci = ChopIngredient(name=ingdir,program_keys=kdict, structure=my_structure)
        myuci.give_saddle_structure("next_ingred") #should be OSZ3
        saddle = MASTFile("%s/writedir/next_ingred/POSCAR" % testdir)
        self.assertEqual(myrelaxed['03'].data, saddle.data)
        saddledir = myuci.get_saddle_dir()
        self.assertEqual(saddledir, "03")
        #self.testclass.give_saddle_structure(childname)

    def test_give_phonon_multiple_forces_and_displacements(self):
        ingdir="%s/writedir/single_phonon_label1" % testdir
        recipedir="%s/writedir" % testdir
        topmetad = MASTFile("files/top_metadata_single")
        topmetad.data.append("origin_dir = %s/files\n" % testdir) #give origin directory
        topmetad.to_file("writedir/metadata.txt")
        metad = MASTFile("files/metadata_single_phonon")
        metad.to_file("%s/metadata.txt" % ingdir)
        mypos = MASTFile("files/phonon_initial_POSCAR")
        mypos.to_file("%s/POSCAR" % ingdir)
        kdict=dict()
        kdict['mast_program'] = 'vasp'
        my_structure = pymatgen.io.vaspio.Poscar.from_file("files/perfect_structure").structure
        myxdat=dict()
        mydynmat=dict()
        for subdir in ['phon_01','phon_02','phon_03']:
            os.mkdir("%s/%s" % (ingdir,subdir))
            myxdat[subdir] = MASTFile("files/XDATCAR_%s" % subdir)
            myxdat[subdir].to_file("%s/%s/XDATCAR" % (ingdir,subdir))
            mydynmat[subdir] = MASTFile("files/DYNMAT_%s" % subdir)
            mydynmat[subdir].to_file("%s/%s/DYNMAT" % (ingdir, subdir))
        myuci = ChopIngredient(name=ingdir,program_keys=kdict, structure=my_structure)
        myuci.give_phonon_multiple_forces_and_displacements("next_ingred") 
        newpos = MASTFile("%s/writedir/next_ingred/POSCAR_prePHON" % testdir)
        newdyn = MASTFile("%s/writedir/next_ingred/DYNMAT" % testdir)
        newxdat = MASTFile("%s/writedir/next_ingred/XDATCAR" % testdir)
        comparepos = MASTFile("files/phonon_initial_POSCAR")
        comparedyn = MASTFile("files/DYNMAT_compare")
        comparexdat = MASTFile("%s/XDATCAR" % ingdir)
        self.assertEqual(newpos.data, comparepos.data)
        self.assertEqual(newdyn.data, comparedyn.data)
        self.assertEqual(newxdat.data, comparexdat.data)
        #self.testclass.give_phonon_multiple_forces_and_displacements(childname)

    def test_give_phonon_single_forces_and_displacements(self):
        ingdir="%s/writedir/single_phonon_label1" % testdir
        recipedir="%s/writedir" % testdir
        topmetad = MASTFile("files/top_metadata_single")
        topmetad.data.append("origin_dir = %s/files\n" % testdir) #give origin directory
        topmetad.to_file("writedir/metadata.txt")
        metad = MASTFile("files/metadata_single_phonon")
        metad.to_file("%s/metadata.txt" % ingdir)
        mypos = MASTFile("files/phonon_initial_POSCAR")
        mypos.to_file("%s/POSCAR" % ingdir)
        kdict=dict()
        kdict['mast_program'] = 'vasp'
        my_structure = pymatgen.io.vaspio.Poscar.from_file("files/perfect_structure").structure
        myxdat = MASTFile("files/XDATCAR_compare")
        myxdat.to_file("%s/XDATCAR" % ingdir)
        mydynmat = MASTFile("files/DYNMAT_compare")
        mydynmat.to_file("%s/DYNMAT" % ingdir)
        myuci = ChopIngredient(name=ingdir,program_keys=kdict, structure=my_structure)
        myuci.give_phonon_single_forces_and_displacements("next_ingred") 
        newpos = MASTFile("%s/writedir/next_ingred/POSCAR_prePHON" % testdir)
        newdyn = MASTFile("%s/writedir/next_ingred/DYNMAT" % testdir)
        newxdat = MASTFile("%s/writedir/next_ingred/XDATCAR" % testdir)
        comparepos = MASTFile("files/phonon_initial_POSCAR")
        comparedyn = MASTFile("files/DYNMAT_compare")
        comparexdat = MASTFile("%s/XDATCAR" % ingdir)
        self.assertEqual(newpos.data, comparepos.data)
        self.assertEqual(newdyn.data, comparedyn.data)
        self.assertEqual(newxdat.data, comparexdat.data)
        #self.testclass.give_phonon_single_forces_and_displacements(childname)

    def test_give_structure_and_energy_to_neb(self):
        ingdir="%s/writedir/single_label1" % testdir
        recipedir="%s/writedir" % testdir
        topmetad = MASTFile("files/top_metadata_single")
        topmetad.data.append("origin_dir = %s/files\n" % testdir) #give origin directory
        topmetad.to_file("writedir/metadata.txt")
        metad = MASTFile("files/metadata_single")
        metad.data.append("defect_label = label1\n")
        metad.to_file("%s/metadata.txt" % ingdir)
        kdict=dict()
        kdict['mast_program'] = 'vasp'
        my_structure = pymatgen.io.vaspio.Poscar.from_file("files/perfect_structure").structure
        myrelaxed = MASTFile("files/relaxed_structure")
        myrelaxed.to_file("%s/CONTCAR" % ingdir)
        myenergy = MASTFile("files/OSZICAR_relaxed")
        myenergy.to_file("%s/OSZICAR" % ingdir)
        myuci = ChopIngredient(name=ingdir,program_keys=kdict, structure=my_structure)
        myuci.give_structure_and_energy_to_neb("next_ingred")
        givenstr = MASTFile("%s/writedir/next_ingred/parent_structure_label1" % testdir)
        givenenergy = MASTFile("%s/writedir/next_ingred/parent_energy_label1" % testdir)
        self.assertEqual(myrelaxed.data, givenstr.data)
        self.assertEqual(myenergy.data, givenenergy.data)
        #self.testclass.give_structure_and_energy_to_neb(childname)

    def test_give_structure_and_restart_files(self):
        ingdir="%s/writedir/single_label1" % testdir
        recipedir="%s/writedir" % testdir
        topmetad = MASTFile("files/top_metadata_single")
        topmetad.data.append("origin_dir = %s/files\n" % testdir) #give origin directory
        topmetad.to_file("writedir/metadata.txt")
        metad = MASTFile("files/metadata_single")
        metad.to_file("%s/metadata.txt" % ingdir)
        kdict=dict()
        kdict['mast_program'] = 'vasp'
        my_structure = pymatgen.io.vaspio.Poscar.from_file("files/perfect_structure").structure
        myrelaxed = MASTFile("files/relaxed_structure")
        myrelaxed.to_file("%s/CONTCAR" % ingdir)
        mychg = MASTFile("files/CHGCAR")
        mychg.to_file("%s/CHGCAR" % ingdir)
        mywave = MASTFile("files/WAVECAR")
        mywave.to_file("%s/WAVECAR" % ingdir)
        myuci = ChopIngredient(name=ingdir,program_keys=kdict, structure=my_structure)
        myuci.give_structure_and_restart_files("next_ingred")
        givenstr = MASTFile("%s/writedir/next_ingred/POSCAR" % testdir)
        self.assertEqual(myrelaxed.data, givenstr.data)
        self.assertTrue(os.path.exists("%s/writedir/next_ingred/WAVECAR" % testdir))
        self.assertTrue(os.path.exists("%s/writedir/next_ingred/CHGCAR" % testdir))
        #self.testclass.give_structure_and_restart_files(childname)

    def test_give_structure_and_restart_files_full_copies(self):
        ingdir="%s/writedir/single_label1" % testdir
        recipedir="%s/writedir" % testdir
        topmetad = MASTFile("files/top_metadata_single")
        topmetad.data.append("origin_dir = %s/files\n" % testdir) #give origin directory
        topmetad.to_file("writedir/metadata.txt")
        metad = MASTFile("files/metadata_single")
        metad.to_file("%s/metadata.txt" % ingdir)
        kdict=dict()
        kdict['mast_program'] = 'vasp'
        my_structure = pymatgen.io.vaspio.Poscar.from_file("files/perfect_structure").structure
        myrelaxed = MASTFile("files/relaxed_structure")
        myrelaxed.to_file("%s/CONTCAR" % ingdir)
        mychg = MASTFile("files/CHGCAR")
        mychg.to_file("%s/CHGCAR" % ingdir)
        mywave = MASTFile("files/WAVECAR")
        mywave.to_file("%s/WAVECAR" % ingdir)
        myuci = ChopIngredient(name=ingdir,program_keys=kdict, structure=my_structure)
        myuci.give_structure_and_restart_files_full_copies("next_ingred")
        givenstr = MASTFile("%s/writedir/next_ingred/POSCAR" % testdir)
        self.assertEqual(myrelaxed.data, givenstr.data)
        self.assertTrue(os.path.exists("%s/writedir/next_ingred/WAVECAR" % testdir))
        self.assertTrue(os.path.exists("%s/writedir/next_ingred/CHGCAR" % testdir))
        #self.testclass.give_structure_and_restart_files(childname)

    def test_give_structure_and_restart_files_softlinks(self):
        ingdir="%s/writedir/single_label1" % testdir
        recipedir="%s/writedir" % testdir
        topmetad = MASTFile("files/top_metadata_single")
        topmetad.data.append("origin_dir = %s/files\n" % testdir) #give origin directory
        topmetad.to_file("writedir/metadata.txt")
        metad = MASTFile("files/metadata_single")
        metad.to_file("%s/metadata.txt" % ingdir)
        kdict=dict()
        kdict['mast_program'] = 'vasp'
        my_structure = pymatgen.io.vaspio.Poscar.from_file("files/perfect_structure").structure
        myrelaxed = MASTFile("files/relaxed_structure")
        myrelaxed.to_file("%s/CONTCAR" % ingdir)
        mychg = MASTFile("files/CHGCAR")
        mychg.to_file("%s/CHGCAR" % ingdir)
        mywave = MASTFile("files/WAVECAR")
        mywave.to_file("%s/WAVECAR" % ingdir)
        myuci = ChopIngredient(name=ingdir,program_keys=kdict, structure=my_structure)
        myuci.give_structure_and_restart_files_softlinks("next_ingred")
        givenstr = MASTFile("%s/writedir/next_ingred/POSCAR" % testdir)
        self.assertEqual(myrelaxed.data, givenstr.data)
        self.assertTrue(os.path.exists("%s/writedir/next_ingred/WAVECAR" % testdir))
        self.assertTrue(os.path.exists("%s/writedir/next_ingred/CHGCAR" % testdir))
        #self.testclass.give_structure_and_restart_files(childname)

    def test_give_structure_and_charge_density_softlink(self):
        ingdir="%s/writedir/single_label1" % testdir
        recipedir="%s/writedir" % testdir
        topmetad = MASTFile("files/top_metadata_single")
        topmetad.data.append("origin_dir = %s/files\n" % testdir) #give origin directory
        topmetad.to_file("writedir/metadata.txt")
        metad = MASTFile("files/metadata_single")
        metad.to_file("%s/metadata.txt" % ingdir)
        kdict=dict()
        kdict['mast_program'] = 'vasp'
        my_structure = pymatgen.io.vaspio.Poscar.from_file("files/perfect_structure").structure
        myrelaxed = MASTFile("files/relaxed_structure")
        myrelaxed.to_file("%s/CONTCAR" % ingdir)
        mychg = MASTFile("files/CHGCAR")
        mychg.to_file("%s/CHGCAR" % ingdir)
        mywave = MASTFile("files/WAVECAR")
        mywave.to_file("%s/WAVECAR" % ingdir)
        myuci = ChopIngredient(name=ingdir,program_keys=kdict, structure=my_structure)
        myuci.give_structure_and_charge_density_softlink("next_ingred")
        givenstr = MASTFile("%s/writedir/next_ingred/POSCAR" % testdir)
        self.assertEqual(myrelaxed.data, givenstr.data)
        self.assertTrue(os.path.exists("%s/writedir/next_ingred/CHGCAR" % testdir))
        #self.testclass.give_structure_and_restart_files(childname)

    def test_give_structure_and_wavefunction_softlink(self):
        ingdir="%s/writedir/single_label1" % testdir
        recipedir="%s/writedir" % testdir
        topmetad = MASTFile("files/top_metadata_single")
        topmetad.data.append("origin_dir = %s/files\n" % testdir) #give origin directory
        topmetad.to_file("writedir/metadata.txt")
        metad = MASTFile("files/metadata_single")
        metad.to_file("%s/metadata.txt" % ingdir)
        kdict=dict()
        kdict['mast_program'] = 'vasp'
        my_structure = pymatgen.io.vaspio.Poscar.from_file("files/perfect_structure").structure
        myrelaxed = MASTFile("files/relaxed_structure")
        myrelaxed.to_file("%s/CONTCAR" % ingdir)
        mychg = MASTFile("files/CHGCAR")
        mychg.to_file("%s/CHGCAR" % ingdir)
        mywave = MASTFile("files/WAVECAR")
        mywave.to_file("%s/WAVECAR" % ingdir)
        myuci = ChopIngredient(name=ingdir,program_keys=kdict, structure=my_structure)
        myuci.give_structure_and_wavefunction_softlink("next_ingred")
        givenstr = MASTFile("%s/writedir/next_ingred/POSCAR" % testdir)
        self.assertEqual(myrelaxed.data, givenstr.data)
        self.assertTrue(os.path.exists("%s/writedir/next_ingred/WAVECAR" % testdir))
        #self.testclass.give_structure_and_restart_files(childname)
    def test_give_structure_and_wavefunction_full_copy(self):
        ingdir="%s/writedir/single_label1" % testdir
        recipedir="%s/writedir" % testdir
        topmetad = MASTFile("files/top_metadata_single")
        topmetad.data.append("origin_dir = %s/files\n" % testdir) #give origin directory
        topmetad.to_file("writedir/metadata.txt")
        metad = MASTFile("files/metadata_single")
        metad.to_file("%s/metadata.txt" % ingdir)
        kdict=dict()
        kdict['mast_program'] = 'vasp'
        my_structure = pymatgen.io.vaspio.Poscar.from_file("files/perfect_structure").structure
        myrelaxed = MASTFile("files/relaxed_structure")
        myrelaxed.to_file("%s/CONTCAR" % ingdir)
        mychg = MASTFile("files/CHGCAR")
        mychg.to_file("%s/CHGCAR" % ingdir)
        mywave = MASTFile("files/WAVECAR")
        mywave.to_file("%s/WAVECAR" % ingdir)
        myuci = ChopIngredient(name=ingdir,program_keys=kdict, structure=my_structure)
        myuci.give_structure_and_wavefunction_full_copy("next_ingred")
        givenstr = MASTFile("%s/writedir/next_ingred/POSCAR" % testdir)
        self.assertEqual(myrelaxed.data, givenstr.data)
        self.assertTrue(os.path.exists("%s/writedir/next_ingred/WAVECAR" % testdir))
        #self.testclass.give_structure_and_restart_files(childname)
    def test_give_structure_and_charge_density_full_copy(self):
        ingdir="%s/writedir/single_label1" % testdir
        recipedir="%s/writedir" % testdir
        topmetad = MASTFile("files/top_metadata_single")
        topmetad.data.append("origin_dir = %s/files\n" % testdir) #give origin directory
        topmetad.to_file("writedir/metadata.txt")
        metad = MASTFile("files/metadata_single")
        metad.to_file("%s/metadata.txt" % ingdir)
        kdict=dict()
        kdict['mast_program'] = 'vasp'
        my_structure = pymatgen.io.vaspio.Poscar.from_file("files/perfect_structure").structure
        myrelaxed = MASTFile("files/relaxed_structure")
        myrelaxed.to_file("%s/CONTCAR" % ingdir)
        mychg = MASTFile("files/CHGCAR")
        mychg.to_file("%s/CHGCAR" % ingdir)
        mywave = MASTFile("files/WAVECAR")
        mywave.to_file("%s/WAVECAR" % ingdir)
        myuci = ChopIngredient(name=ingdir,program_keys=kdict, structure=my_structure)
        myuci.give_structure_and_charge_density_full_copy("next_ingred")
        givenstr = MASTFile("%s/writedir/next_ingred/POSCAR" % testdir)
        self.assertEqual(myrelaxed.data, givenstr.data)
        self.assertTrue(os.path.exists("%s/writedir/next_ingred/CHGCAR" % testdir))
        #self.testclass.give_structure_and_restart_files(childname)



##############################################################
# This code is part of the MAterials Simulation Toolkit (MAST)
# 
# Maintainer: Glen Jenness
# Last updated: 2014-07-01
##############################################################
import os
import numpy as np
from MAST.utility import MASTError

class InputOptions:
    """Stores options information in dictionary format for retrieval
        Dictionary is organised as sections and key, value pairs 
            within a section
        Attributes:
            options <dict of sections which is a dict of key/values>: 
                stores options which consist of sections and key/values 
                under the sections
        e.g.: section mast, keyword 'program'
                inputoptions.options['mast']['program'] = 'vasp'
    """

    def __init__(self):
        """Constructor for InputOptions
        Initialize the dictionary
        """
        self.options = dict()

    def set_item(self, section, key, value):
        """Adds the key/value dict item into the option under the 
        respective section.
            NO CHANGE if the key was already in the section.
        """
        section_dict = self.options.setdefault(section, dict())
        if 'path' in key:
	        section_dict.setdefault(key, os.path.expanduser(value))
        else:
	        section_dict.setdefault(key, value)

    def get_item(self, section, key=None, default_value=None):
        """Returns the value of the key under this section
        """
        if key is None:
            return self.options.get(section, dict())
        else:
            return self.options.get(section, dict()).get(key, None)

    def update_item(self, section, key, value):
        """Updates the key/value dict item in the option."""
        section_dict = self.options.setdefault(section, dict())
        if key in section_dict.keys():
            print "Changing value of %s, %s." % (section, key)
        if 'path' in key:
            section_dict.__setitem__(key, os.path.expanduser(value))
        else:
            section_dict.__setitem__(key, value)


    def get_sections(self):
        """Return the available sections in the input options.
        """
        return self.options.keys()

    def get_section_keys(self, section):
        """Return the available keys under a section."""
        if not section in self.get_sections():
            raise MASTError(self.__class__.__name__,
                section + " was not parsed from the input file.")
        return self.options[section].keys()

    def reset(self):
        """Option to reset the dict values
        """
        self.options = dict()

    def __repr__(self):
        rlines=""
        rlines=rlines + "Input options: \n"
        sections = self.get_sections()
        sections.sort()
        for section in sections:
            rlines = rlines + "*********************\n"
            rlines = rlines + "*   %s section\n" % section
            rlines = rlines + "*********************\n"
            skeys = self.get_section_keys(section)
            skeys.sort()
            for skey in skeys:
                rlines = rlines + "------------------\n" 
                rlines = rlines + "%s:\n" % skey
                rlines = rlines + "------------------\n" 
                rlines = rlines + "    %s\n" % str(self.get_item(section,skey))
        return rlines
    


    def print_python_section(self, secname):
        """Print python commands that will recreate the InputOptions section
            when the commands are run in python.

            secname <str>: section name
        """
        pln=list()
        if not type(secname) == str:
            errorstr = "Section name " + str(secname) + "must be a string."
            raise MASTError(self.__class__.__name__, errorstr)
        if not secname in self.options.keys():
            errorstr = "No section " + str(secname) + "in self.options."
            raise MASTError(self.__class__.__name__, errorstr)
        initheader=secname
        pln.append(initheader + " = dict()")
        for key1, val1 in self.options[secname].iteritems():
            if key1 == 'structure':
                pln.append("#ignoring created structure")
                continue
            if type(key1) == str:
                header = initheader + "[" + "'" + key1 + "'" + "]"
            else:
                header = initheader + "[" + key1 + "]"
            if not type(key1) == str:
                errorstr = "One of the keys in section names is not a string."
                raise MASTError(self.__class__.__name__, errorstr)
            if type(val1) == str:
                mystr = header + " = " + "'" + val1 + "'"
                pln.append(mystr)
            elif type(val1) in [int, bool, float]:
                mystr = header + " = " + str(val1)
                pln.append(mystr)
            elif val1 == None:
                mystr = header + " = None"
                pln.append(mystr)
            elif type(val1) == np.ndarray:
                pln.append(header + "=" + "np.array("+str(val1.tolist())+")")
            elif type(val1) == list:
                mystr = header + " = list()"
                pln.append(mystr)
                for myitem in val1:
                    if type(myitem) == str:
                        mystr = header + ".append(" + "'" + myitem + "'" + ")"
                        pln.append(mystr)
                    else:
                        pln.append(header + ".append(" + str(myitem) + ")")
            elif type(val1) == dict:
                mystr = header + " = dict()"
                pln.append(mystr)
                pln.extend(self.print_secondlevel_dict(header, val1))
            else:
                errstr="UNSUPPORTED TYPE" + str(type(val1))
                raise MASTError(self.__class__.__name__, errstr)
        return pln

    def print_secondlevel_dict(self, initheader1, seconddict):
        """Print a second-level dictionary. 
            
            initheader1 <str>: prepending header
            seconddict <dict>: dictionary that was key1's value
        """
        pln=list()
        for key2, val2 in seconddict.iteritems():
            if type(key2) == str:
                header = initheader1 + "[" + "'" + key2 + "'" + "]"
            else:
                header = initheader1 + "[" + key2 + "]"
            if type(val2) == str:
                mystr = header + " = " + "'" + val2 + "'"
                pln.append(mystr)
            elif type(val2) in [int,float,bool]:
                pln.append(header + "=" + str(val2))
            elif val2 == None:
                mystr = header + " = None"
                pln.append(mystr)
            elif type(val2) == np.ndarray:
                pln.append(header + "=" + "np.array(" + str(val2.tolist()) +")")
            elif type(val2) == list:
                mystr = header + " = list()"
                pln.append(mystr)
                for myitem in val2:
                    if type(myitem) == str:
                        mystr = header + ".append(" + "'" + myitem + "'" + ")"
                        pln.append(mystr)
                    else:
                        pln.append(header + ".append(" + str(myitem) +")")
            elif type(val2) == dict:
                mystr = header + " = dict()"
                pln.append(mystr)
                pln.extend(self.print_thirdlevel_dict(header, val2))
            else:
                errstr="UNSUPPORTED TYPE" + str(type(val2))
                raise MASTError(self.__class__.__name__, errstr)
        return pln

    def print_thirdlevel_dict(self, initheader2, thirddict):
        """Print a third-level dictionary. 
            
            initheader2 <str>: prepending header
            thirddict <dict>: dictionary that was key2's value
        """
        pln=list()
        for key3, val3 in thirddict.iteritems():
            if type(key3) == str:
                header = initheader2 + "[" + "'" + key3 + "'" + "]"
            else:
                header = initheader2 + "[" + key3 + "]"
            if type(val3) == str:
                mystr = header + " = " + "'" + val3 + "'"
                pln.append(mystr)
            elif type(val3) in [int, float, bool]:
                pln.append(header + " = " + str(val3))
            elif val3 == None:
                mystr = header + " = None"
                pln.append(mystr)
            elif type(val3) == np.ndarray:
                pln.append(header + "=" + "np.array(" + str(val3.tolist()) +")")
            elif type(val3) == list:
                mystr = header + " = list()"
                pln.append(mystr)
                for myitem in val3:
                    if type(myitem) == str:
                        mystr = header + ".append(" + "'" + myitem + "'" + ")"
                        pln.append(mystr)
                    else:
                        pln.append(header + ".append(" + str(myitem) +")")
            elif type(val3) == dict:
                mystr = header + " = dict()"
                pln.append(mystr)
                pln.extend(self.print_fourthlevel_dict(header, val3))
            else:
                errstr="UNSUPPORTED TYPE" + str(type(val3))
                raise MASTError(self.__class__.__name__, errstr)
        return pln
    
    def print_fourthlevel_dict(self, initheader2, thirddict):
        """Print a third-level dictionary. 
            
            initheader2 <str>: prepending header
            thirddict <dict>: dictionary that was key2's value
        """
        pln=list()
        for key3, val3 in thirddict.iteritems():
            if type(key3) == str:
                header = initheader2 + "[" + "'" + key3 + "'" + "]"
            else:
                header = initheader2 + "[" + key3 + "]"
            if type(val3) == str:
                mystr = header + " = " + "'" + val3 + "'"
                pln.append(mystr)
            elif type(val3) in [int, float, bool]:
                pln.append(header + " = " + str(val3))
            elif val3 == None:
                mystr = header + " = None"
                pln.append(mystr)
            elif type(val3) == np.ndarray:
                pln.append(header + "=" + "np.array(" + str(val3.tolist()) +")")
            elif type(val3) == list:
                mystr = header + " = list()"
                pln.append(mystr)
                for myitem in val3:
                    if type(myitem) == str:
                        mystr = header + ".append(" + "'" + myitem + "'" + ")"
                        pln.append(mystr)
                    else:
                        pln.append(header + ".append(" + str(myitem) +")")
            elif type(val3) == dict:
                mystr = header + " = dict()"
                pln.append(mystr)
                pln.append(header + "=" + str(val3))
            else:
                errstr="UNSUPPORTED TYPE" + str(type(val3))
                raise MASTError(self.__class__.__name__, errstr)
        return pln



from MAST.structopt.tools.find_defects import find_defects
from ase import Atom, Atoms

def rattle(indiv):
    """Function to slightly alter atoms in structure. Intended for use in defect function.
    """
    atms,indb,vacant,swap,stro = find_defects(indiv[0],self.solidbulk,0.0)
    atmsl,indbl,vacantl,swapl,strol = find_defects(indiv[0],self.solidbulk,2.0)
    atmslist = []
    for atm1 in atmsl:
        for atm2 in atms:
            if atm1.symbol==atm2.symbol:
                if atm1.position[0]==atm2.position[0] and atm1.position[1]==atm2.position[1] and atm1.position[2]==atm2.position[2]:
                    atmslist.append(atm1.index)
    atmolist=[atm for atm in atmsl if atm.index not in atmslist]
    rat=Atoms(cell=atms.get_cell(), pbc=atms.get_pbc)
    for one in atmolist:
        rat.append(one)
    rat.rattle(stdev=0.2)
    ind=atms.copy()
    ind.extend(rat)
    ind.extend(indbl)
    return ind
	
	


#!/usr/bin/env python
##############################################################
# This code is part of the MAterials Simulation Toolkit (MAST)
# 
# Maintainer: Wei Xie
# Last updated: 2014-05-16
##############################################################
import sys
import math
import warnings
from collections import OrderedDict

class writer :
        def __init__(self, *writers) :
                self.writers = writers

        def write(self, text) :
                for w in self.writers :
                        w.write(text)

def roof_mean(intA,intB):
    """
    This is a helper method that calculates the integer mean of two integers 
    rounded to roof if the direct mean of the two integers are not a integer.
    """
    if (intA+intB)%2 == 0:
        return (intA+intB)/2
    else:
        return (intA+intB+1)/2

def branchingLR(inputList,outputList,m):
    """
    inputList must be a lexicographically ordered list of real numbers with no
    duplicates, and len(inputList)>=3. 
    Note when len(inputlist)==3, the output for left and right branches will be
    the same. 
    m is the number of expected elemented to be picked, m >=3. 
    """    
    if len(inputList)<3:
        raise RuntimeError("The input list only has "+str(len(inputList))+
        " elements, but this algorithm requires it to have >=3 elements.")  
    elif m<3:
        raise RuntimeError("Your input paramter m is "+str(m)+
        " , but this algorithm requires it to be >=3.")     
            
    listMin=inputList[0]
    listMax=inputList[len(inputList)-1] 
           
    Step=(listMax-listMin)/(m-1)

    ####---------min end---------------####              
    for i in range(1,len(inputList),1):
        if inputList[i]>listMin+Step:
            break
            
    if i==1 or abs(inputList[i]-(listMin+Step))<abs(inputList[i-1]-(listMin+Step)):
        leftPick=inputList[i]
        newInputListL=inputList[i:len(inputList)]
    else:        
        leftPick=inputList[i-1]
        newInputListL=inputList[(i-1):len(inputList)]
        
    newOutputListL=list(outputList)
    newOutputListL.append(leftPick)     
        
    ####---------max end---------------####             
    for i in range((len(inputList)-2),0,-1):
        if inputList[i]<listMax-Step:
            break

    if i==len(inputList)-2 or abs(inputList[i]-(listMax-Step))<abs(inputList[i+1]-(listMax-Step)):
        rightPick=inputList[i]
        newInputListR=inputList[0:(i+1)]
    else:
        rightPick=inputList[i+1]
        newInputListR=inputList[0:((i+1)+1)] 

    newOutputListR=list(outputList)
    newOutputListR.append(rightPick)     
        
    return newInputListL,newOutputListL,newInputListR,newOutputListR

def pickSubList(inputList,m):
    
    
    if len(inputList)<m:
        raise RuntimeError("The input list only has "+str(len(inputList))+
        " elements, but you asked to pick "+str(m)+" elements out of the list!")
    elif len(inputList)==m:
        return sorted(list(OrderedDict.fromkeys(inputList))) 
    elif len(inputList)<3:
        raise RuntimeError("The input list only has "+str(len(inputList))+
        " elements, but this algorithm requires it to have >=3 elements.")        
    elif type(m) != int or m<3:
        raise RuntimeError("You asked to select "+str(m)+
        " elements, but this algorithm requires it to be >=3.")        
      

    #removes duplicates from and order the list      
    inputList=sorted(list(OrderedDict.fromkeys(inputList)))
    
    outputList=[inputList[0],inputList[len(inputList)-1]] 

    allInputListDic={}
    allOutputListDic={}
    
    allInputListDic[str(0)+str(0)]=inputList
    allOutputListDic[str(0)+str(0)]=outputList
            
    for i in range(m-2):
        #print "i is "+ str(i)
        for j in range(2**i):
            #print "j is "+ str(j) 
            if len(allInputListDic[str(i)+str(j)])>=3:           
                (newInputListL,newOutputListL,
                 newInputListR,newOutputListR)=branchingLR(allInputListDic[str(i)+str(j)],
                                                       allOutputListDic[str(i)+str(j)],m-i)
            else:
                newInputListL=allInputListDic[str(i)+str(j)]
                newOutputListL=allOutputListDic[str(i)+str(j)]
                newInputListR=allInputListDic[str(i)+str(j)]
                newOutputListR=allOutputListDic[str(i)+str(j)]             
            
            allInputListDic[str(i+1)+str(2*j+0)]=newInputListL
            allOutputListDic[str(i+1)+str(2*j+0)]=newOutputListL
            allInputListDic[str(i+1)+str(2*j+1)]=newInputListR
            allOutputListDic[str(i+1)+str(2*j+1)]=newOutputListR         
  
    bestUniformity=sys.float_info.max
    bestOutputList=[]
    idealSpacing=(inputList[len(inputList)-1]-inputList[0])/(m-1)
    i=m-2
    while bestOutputList==[]:
        for j in range(2**(i-1)):
            for k in [0,1]:
                dummyList=allOutputListDic[str(i)+str(2*j+k)]
                if len(dummyList) == m:
                    dummyUniformity=0
                    for l in range(len(dummyList)-1):
                        dummyUniformity=dummyUniformity+(
                            abs(dummyList[l+1]-dummyList[l])-idealSpacing)**2
                    dummyUniformity=math.sqrt(dummyUniformity/(len(dummyList)-1))                    
                    if dummyUniformity<bestUniformity:
                        bestOutputList=dummyList
        i=i-1
    
    if len(bestOutputList)<m:
        warnings.warn("You asked to pick "+str(m)+" elments out of the inputlist, \
                       but this algorithm only managed to pick"+str(len(bestOutputList))+
                      " elements. The intputlist is probably a very non-uniformly \
                       distributed list.")
                    
    return sorted(bestOutputList)    
 

if __name__ == "__main__":

    saved = sys.stdout
    fout = file('out.log', 'w')
    sys.stdout = writer(sys.stdout, fout)

    testlist=[9.67,6.16,2.59,-0.98,5.0,3.24,1.46,2.24,1.07,0.27,4.84, 
              3.97,3.08,3.48,2.9,2.5,3.22,2.84,2.42]
    
    for n in range(3,21):
        print pickSubList(testlist,n)
         
    sys.stdout = saved
    fout.close()                


import random
import numpy
from ase import Atom, Atoms
from MAST.structopt.tools.find_defects import find_defects

def rotation_geo(indiv, Optimizer):
    '''Function to handle a geometry based rotation mutation
    Rotates a small group of atoms based on location within structure
    Input:
        indiv = structopt Individual class object to be mutated
        Optimizer = structopt Optimizer class object
    Output:
        indiv = structopt Individual class object that has been mutated
    '''
    if 'MU' in Optimizer.debug:
        debug = True
    else:
        debug = False
    #Rotate group of atoms based on geometry
    if Optimizer.structure=='Defect':
        if Optimizer.isolate_mutation:
            atms,indb,vacant,swap,stro = find_defects(indiv[0],Optimizer.solidbulk,0)
        else:
            atms = indiv[0].copy()
    else:
        atms=indiv[0].copy()
    nat=len(atms)
    if nat > 1:
        #Select number of atoms to rotate
        if nat==2:
            natrot=2
        else:
            natrot = random.randint(2,nat)
        #Select random position in cluster
        cell=numpy.maximum.reduce(atms.get_positions())-numpy.minimum.reduce(atms.get_positions())
        pt=[random.uniform(0,cell[0]),random.uniform(0,cell[1]),random.uniform(0,cell[2])]
        #Get distance of atoms from random point
        atpt=Atom(position=pt)
        atms.append(atpt)
        dist=[]
        for i in range(len(atms)-1):
            dist.append(atms.get_distance(i,len(atms)-1))
        atms.pop()
        dlist=zip(dist,atms)
        dlist=sorted(dlist, key=lambda one: one[0], reverse=True)
        # Select atoms closest to random point
        atmsr=Atoms()
        indexlist=[]
        for i in range(natrot):
            atmsr.append(dlist[i][1])
            indexlist.append(dlist[i][1].index)
        #Identify random rotation and rotate selected atoms
        ax=['x', '-x','y','-y','z','-z']
        rax=ax[random.randint(0,len(ax)-1)]
        rang=random.uniform(30,180)
        #rang=random.random()*90
        atmsr.rotate(rax,a=rang,center='COM',rotate_cell=False)
        #Update individual with new atom positions
        for i in range(len(indexlist)):
            index=indexlist[i]
            atms[index].position=atmsr[i].position
        if Optimizer.structure=='Defect':
            if Optimizer.isolate_mutation:
                atms.extend(indb)
        indiv[0]=atms.copy()
        #Optimizer.output.write('New positions = '+repr(indiv[0].get_positions())+'\n')
    else:
        natrot=0
        pt=0
        rax=0
        rang=0
    Optimizer.output.write('Geometry Rotation Mutation performed on individual\n')
    Optimizer.output.write('Index = '+repr(indiv.index)+'\n')
    Optimizer.output.write('Number of atoms rotated = '+repr(natrot)+'\n')
    Optimizer.output.write('Geometry point = '+repr(pt)+'\n')
    Optimizer.output.write('Rotation vector = '+repr(rax)+'\n')
    Optimizer.output.write('Rotation angle = '+repr(rang)+'\n')
    Optimizer.output.write(repr(indiv[0])+'\n')
    muttype='GR'+repr(natrot)
    if indiv.energy==0:
        indiv.history_index=indiv.history_index+'m'+muttype
    else:
        indiv.history_index=repr(indiv.index)+'m'+muttype
    return indiv


##############################################################
# This code is part of the MAterials Simulation Toolkit (MAST)
# 
# Maintainer: Tam Mayeshiba
# Last updated: 2014-04-25
##############################################################
import logging
import os
import time
def validate_recipe_name(rname):
    """Validate the recipe name.
        Use $MAST_CONTROL if recipe name is not found in $MAST_SCRATCH.
        Args:
            rname <str>: Recipe name candidate
        Returns:
            rnamevalid <str>: Validated recipe name
    """
    #check existence of rname:
    mast_scratch = os.getenv("MAST_SCRATCH")
    rdirs = os.listdir(mast_scratch)
    rnamevalid = os.getenv("MAST_CONTROL")
    for rdir in rdirs:
        if rdir in rname:
            rnamevalid = os.path.join(mast_scratch, rdir)
            return rnamevalid
    return rnamevalid

def config_for_recipe_and_control(rname):
    rnamevalid = validate_recipe_name(rname)
    formatstr ='%(asctime)s : %(module)10s: %(levelname)8s : %(message)s'
    longformatter = logging.Formatter(formatstr)
    formatstrshort ='%(levelname)8s : %(message)s'
    shortformatter = logging.Formatter(formatstrshort)
    recipehandler = logging.FileHandler(filename="%s/mast_recipe.log" % rnamevalid)
    recipehandler.setFormatter(longformatter)
    controlhandler = logging.FileHandler(filename="%s/mast.log" % os.getenv("MAST_CONTROL"))
    controlhandler.setFormatter(shortformatter)
    logger=logging.getLogger('')
    if not getattr(logger, 'has_recipe_handler', None):
        logger.addHandler(recipehandler)
        logger.has_recipe_handler = True
    if not getattr(logger, 'has_control_handler', None):
        logger.addHandler(controlhandler)
        logger.has_control_handler = True
    logger.setLevel(logging.INFO)
    return logger

def add_handler_for_recipe(rname, logger):
    """Add a handler for the recipe.
        Args:
            rname <str>: Recipe full path
            logger <logging.Logger>: Logger
    """
    #print "Handler added for rname %s at %s" % (rname, time.asctime())
    logger.setLevel(logging.INFO)
    rnamevalid = validate_recipe_name(rname)
    formatstr ='%(asctime)s : %(module)10s: %(name)20s: %(levelname)8s : %(message)s'
    formatter = logging.Formatter(formatstr)
    recipehandler = logging.FileHandler(filename="%s/mast_recipe.log" % rnamevalid)
    recipehandler.setFormatter(formatter)
    rnameshort = os.path.basename(rnamevalid)
    recipefilter = RecipeFilter(rnameshort)
    recipehandler.addFilter(recipefilter)
    if not getattr(logger, 'has_recipe_handlers', None):
        logger.has_recipe_handlers = list()
    if not rnameshort in logger.has_recipe_handlers:
        logger.addHandler(recipehandler)
        logger.has_recipe_handlers.append(rnameshort)
    return logger

def add_handler_for_control(logger):
    """Add a handler for control
        Args:
            logger <logging.Logger>: Logger
    """
    logger.setLevel(logging.INFO)
    #formatstr ='%(levelname)8s : %(name)10s: %(message)s'
    formatstr ='%(levelname)8s : %(message)s'
    formatter = logging.Formatter(formatstr)
    controlhandler = logging.FileHandler(filename="%s/mast.log" % os.getenv("MAST_CONTROL"))
    controlhandler.setFormatter(formatter)
    controlfilter = ControlFilter()
    controlhandler.addFilter(controlfilter)
    if not getattr(logger, 'has_control_handler', None):
        logger.addHandler(controlhandler)
        logger.has_control_handler = True
    return logger

class ControlFilter(logging.Filter):
    def filter(self, record):
        return True
        if getattr(record, 'name', None) in ['mast','mastmon']:
            return True
        else:
            return False

class RecipeFilter(logging.Filter):
    def __init__(self, rnameshort):
        """Recipe filter
            Args:
                rnameshort <str>: Recipe name (not full path)
        """
        logging.Filter.__init__(self)
        self.rname = rnameshort

    def filter(self, record):
        if self.rname in getattr(record, 'name', None):
            return True
        else:
            return False
        

def initialize_logger(filename="default.log", formatstr=""):
    logger     = logging.getLogger(filename)
    if not getattr(logger, 'handler_set', None):
        handler    = logging.FileHandler(filename)
        if formatstr == "":
            #formatter  = logging.Formatter('%(asctime)s : %(module)15s:%(lineno)4d> : %(levelname)8s : %(message)s')
            formatter  = logging.Formatter('%(asctime)s : %(module)10s: %(levelname)8s : %(message)s')
        else:
            formatter = logging.Formatter(formatstr)
        handler.setFormatter(formatter)
        logger.addHandler(handler)
        logger.setLevel(logging.INFO)
        logger.handler_set = True
    return logger

def initialize_short_logger(filename="default.log"):
    format = "%(levelname)8s: %(message)s"
    #format = "%(asctime)s %(levelname)8s : %(message)s"
    return initialize_logger(filename, format)



##############################################################
# This code is part of the MAterials Simulation Toolkit (MAST)
# 
# Maintainer: Tam Mayeshiba
# Last updated: 2014-04-25
##############################################################
import os
import sys
from MAST.utility import Metadata
from MAST.utility import dirutil
from MAST.ingredients.checker import VaspChecker
def main(ingname=""):
    """Get the energy from an ingredient.
        Args:
            ingname <str>: Ingredient name, full path
        Returns:
            <string>: "energy (eV);<energy as a string>", ex:
                "energy (eV); 3.0"
                Returns last E0 energy for a VASP run.
                Returns last E0 energy for all images for a VASP neb.
                Returns "N/A" otherwise.
    """
    trymeta = "%s/metadata.txt" % ingname
    if os.path.isfile(trymeta):
        mymeta = Metadata(metafile=trymeta)
        myprogram = mymeta.read_data("program")
    else:
        myprogram = "None"
    if 'induce' in ingname: #skip inducedefect ingredients
        myprogram = "None"
    if 'vasp' in myprogram:
        if os.path.isdir("%s/01" % ingname):
            estr = "energies (eV)"
            for subdir in dirutil.immediate_subdirs(ingname):
                mychecker = VaspChecker(name="%s/%s" % (ingname, subdir))
                estr = estr + ";%3.3f" % mychecker.get_energy_from_energy_file()
            return estr
        else:
            mychecker = VaspChecker(name=ingname)
            return "energy (eV);%3.3f" % mychecker.get_energy_from_energy_file()
    else:
        return "energy (eV);N/A"



##############################################################
# This code is part of the MAterials Simulation Toolkit (MAST)
# 
# Maintainer: Zhewen Song
# Last updated: 2014-04-25
##############################################################
import pymatgen as mg
from numpy import *
from MAST.utility import create_paths
import os, getopt, sys, re

argv = sys.argv[1:]
opts, args = getopt.getopt(argv,"i","n")
inp = os.path.join(os.getcwd(),args[0])
Nth = int(args[2])
info=create_paths.main(inp,Nth)
[vec,xyz,path]=info

def add_structure():
    """Adding structures to the temporary MAST input file.
    
    """
    fp=open('temp.inp','w+')
    rp=open(inp,'r+').readlines()
    i=0
    while not 'begin lattice' in rp[i]: i=i+1
    while not 'end' in rp[i+1]: del rp[i+1]
    for j in range(3):
        i=i+1; rp.insert(i,str(vec[j][0])+' '+str(vec[j][1])+' '+str(vec[j][2])+' \n')   
    while not 'begin coordinates' in rp[i]: i=i+1
    while not 'end' in rp[i+1]: del rp[i+1]
    for ele in xyz.keys():
        for j in range(len(xyz[ele])):
            i=i+1; rp.insert(i,ele+' '+str(xyz[ele][j][0])+' '+str(xyz[ele][j][1])+' '+str(xyz[ele][j][2])+'\n')
    while not '$site' in rp[i]: i=i+1
    del rp[i]
    while not '$end' in rp[i]: del rp[i]
    fp.writelines(rp)
    
def add_defects():
    """Adding defects with NEB, phonon information to the temporary MAST input file.
    
    """
    ends=[]
    tmp=[]
    dash=re.compile(r'-')
    digit=re.compile(r'\d')
    point=re.compile(r'\.')
    for i in range(len(path['T'])):
        for j in [2,3]:
            if not path['T'][i][j] in ends:
                ends.append(path['T'][i][j])
                name=dash.split(path['T'][i][0])[j-2]
                tmp.append(name)
    types=tmp[:]
    types[0]=tmp[0]+chr(ord('a'))
    for i in range(1,len(tmp)):
        flag=0
        for j in range(i-1,-1,-1):
            if tmp[j]==types[i]:
                types[i]=tmp[i]+chr(ord(digit.split(types[j])[1])+1)
                flag=1
                break
        if flag==0: 
            types[i]=tmp[i]+chr(ord('a')) # New type
    name=point.split(inp)[0]+'_new.inp'    
    fp=open(name,'w+')
    rp=open('temp.inp','r+').readlines()
    rp.append('$defects\nthreshold 1e-4\ncoord_type fractional\n')
    for i in range(len(ends)):   
        rp.append('begin '+types[i]+'\n')
        rp.append('interstitial '+str(ends[i][0])+' '+str(ends[i][1])+' '+str(ends[i][2])+' '+'X'+str(len(xyz.keys())+1)+'\n')
        rp.append('end\n')
    rp.append('$end\n\n')
    rp.append('$neb\n')
    for i in range(len(path['T'])):
        for j in range(len(ends)):
            if ends[j]==path['T'][i][2]:
                start=j
            if ends[j]==path['T'][i][3]:   
                finish=j
        rp.append('begin '+types[start]+'-'+types[finish]+' #'+str(path['T'][i][1])+'NN\n')
        rp.append('X'+str(len(xyz.keys())+1)+', '+str(ends[start][0])+' '+str(ends[start][1])+' '+str(ends[start][2])+', '+str(ends[finish][0])+' '+str(ends[finish][1])+' '+str(ends[finish][2])+'\n')
        rp.append('images 1\n')
        mid=[0,0,0]
        for k in range(3): 
            mid[k]=(ends[start][k]+ends[finish][k])*0.5
            if abs(ends[start][k]-ends[finish][k])>0.5:
                mid[k]=mid[k]-0.5
            
        rp.append('phonon int '+str(mid[0])+' '+str(mid[1])+' '+str(mid[2])+' 0.0\n')
        rp.append('end\n')
    rp.append('$end\n')
    os.system('rm temp.inp')  
    fp.writelines(rp)
    return name

add_structure()
name=add_defects()
os.system('mast -i '+name)



import random
from MAST.structopt.tools.find_defects import find_defects

def permutation_bulk(indiv, Optimizer):
    """Move function to perform Permutation of one atom based on atomlist
    Inputs:
        indiv = Individual class object to be altered
        Optimizer = Optimizer class object with needed parameters
    Outputs:
        indiv = Altered Individual class object
    """
    if 'MU' in Optimizer.debug:
        debug = True
    else:
        debug = False
    if Optimizer.structure != 'Defect':
        Optimizer.output.write('WARNING: Crystal extended permutation attempted on non-Defect system. Skipping Mutation\n')
    else:
        a1=indiv.bulki[random.randint(0,indiv.bulki.get_number_of_atoms()-1)]
        opts=[inds for inds in indiv.bulki if inds.symbol !=a1.symbol]
        try:
            a2=opts[random.randint(0,len(opts)-1)]
        except ValueError:
            a2=indiv.bulki[random.randint(0,indiv.bulki.get_number_of_atoms()-1)]
            Optimizer.output.write('WARNING: Permutation Mutation attempted on single atom structure system\n')
        indiv.bulki[a1.index].symbol, indiv.bulki[a2.index].symbol=a2.symbol, a1.symbol
        Optimizer.output.write('Bulk Permutation Mutation performed on individuals bulk\n')
        Optimizer.output.write('Index = '+repr(indiv.index)+'\n')
        Optimizer.output.write(repr(a1.symbol) + ' swapped with ' + repr(a2.symbol)+'\n')
        Optimizer.output.write(repr(indiv.bulki)+'\n')
        muttype='PB'
        if indiv.energy==0:
            indiv.history_index=indiv.history_index+'m'+muttype
        else:
            indiv.history_index=repr(indiv.index)+'m'+muttype
    return indiv


from MAST.structopt.tools import eval_energy
from MAST.structopt.inp_out.write_xyz import write_xyz
import logging
import math
import os
import numpy

def surfaceenergy(indiv, Optimizer):
    '''Function to calculate surface energy fitness of individual.
    Input:
        indiv = structopt Individual class object to be evaluated
        Optimizer = structopt Optimizer class object
    Output:
        indiv = structopt Individual class object with new fitness.
    '''
    logger = logging.getLogger(Optimizer.loggername)
    #logger = initialize_logger(Optimizer.loggername)
    starting = indiv.duplicate()
    cwd = os.getcwd()
    try:
        outs = eval_energy(Optimizer,indiv)
    except Exception, e:
        logger.warn('Error in energy evaluation: {0}'.format(e), exc_info=True)
        stro = 'ERROR: Problem in Energy Evaluation'
        print stro
        print e
        stro += '\n' + repr(e)
        os.chdir(cwd)
        f=open('problem-structures.xyz','a')
        totalsol = indiv[0].copy()
        totalsol.extend(indiv.bulki)
        write_xyz(f,totalsol,data='Starting structure hindex={0}'.format(indiv.history_index))
        indiv.energy = 10000
        f.close()
        print '    Writing structure to problemstructures.xyz file. Structure (hindex) : '+indiv.history_index
        print '    Setting individual energy to 50000.'
        outs = [10000, starting.bulki, starting, stro]
    indiv.energy = outs[0]
    stro=outs[3]
    if Optimizer.structure == 'Defect' or Optimizer.structure=='Surface':
        indiv.bulki = outs[1]
    fit=indiv.energy
    for sym,c,m,u in Optimizer.atomlist:
        nc=len([atm for atm in indiv[0] if atm.symbol==sym])
        fit-= float(nc)*float(u)
    cel=numpy.maximum.reduce(indiv[0].get_cell())
    Ar=cel[0]*cel[1]
    fit=fit/Ar
    if abs(fit) > Optimizer.energy_cutoff_factor*(len(indiv[0])+len(indiv.bulki)):
        fit=10000
        message = 'Warning: Found oddly large energy from Lammps in structure HI={0}'.format(indiv.history_index)
        logger.warn(message)
        print message
        print '    Setting fitness to 10000'
    if math.isnan(fit):
        logger.warn('Found NAN energy structure HI={0}'.format(indiv.history_index))
        fit=10000
        indiv.energy = 10000
    indiv.fitness=fit
    return indiv, stro


import random

def swap(indiv, Optimizer):
    """Move function to perform atom structure swap based on atomlist
    Inputs:
        indiv = Individual class object to be altered
        Optimizer = Optimizer class object with needed parameters
    Outputs:
        indiv = Altered Individual class object
    """
    if 'MU' in Optimizer.debug:
        debug = True
    else:
        debug = False
    Optimizer.output.write('Swap Mutation performed on individual\n')
    Optimizer.output.write('Index = '+repr(indiv.index)+'\n')
    if len(indiv[0]) > 4:
        natomsswap=random.randint(1,len(indiv[0])/5)
    else:
        natomsswap=1
    Optimizer.output.write('Number of swaps = '+repr(natomsswap)+'\n')
    syms=list(set(indiv[0].get_chemical_symbols()))
    if len(syms)<len(Optimizer.atomlist):
        syms=[sym for sym,c,m,u in Optimizer.atomlist]
    if len(syms)==1:
        Optimizer.output.write('WARNING: Swap Mutation attempted on single atom structure system\n')
    else:
        for i in range(natomsswap):
            if len(indiv[0])>1:
                a1=indiv[0][random.randint(0,indiv[0].get_number_of_atoms()-1)]
            else:
                a1=indiv[0][0]
            osym=a1.symbol
            nsymlist=[sym for sym in syms if sym != osym]
            a1.symbol=random.choice(nsymlist)
            nsym=a1.symbol
            Optimizer.output.write('Swapped '+osym+' atom with '+nsym+'\n')
    Optimizer.output.write(repr(indiv[0])+'\n')
    muttype='S'+repr(natomsswap)
    if indiv.energy==0:
        indiv.history_index=indiv.history_index+'m'+muttype
    else:
        indiv.history_index=repr(indiv.index)+'m'+muttype
    return indiv


import random

def swap_int(indiv, Optimizer):
    """Move function to perform an atom structure swap based on swaplist for cluster
    Inputs:
        indiv = Individual class object to be altered
        Optimizer = Optimizer class object with needed parameters
    Outputs:
        indiv = Altered Individual class object
    """
    if 'MU' in Optimizer.debug:
        debug = True
    else:
        debug = False
    Optimizer.output.write('Cluster Swap Mutation performed on individual\n')
    Optimizer.output.write('Index = '+repr(indiv.index)+'\n')
    swapmax=sum([c for sym,c in indiv.swaplist])
    if swapmax >1:
        natomsswap=random.randint(1,swapmax)
    elif swapmax==1:
        natomsswap=1
    else:
        Optimizer.output.write('Maximum number of swaps is '+repr(swapmax)+'. Unable to perform mutation')
        natomsswap=0
    if len(indiv.swaplist)==1:
        Optimizer.output.write('WARNING: Swap Mutation attempted on single atom structure system\n')
        natomsswap=0
    Optimizer.output.write('Number of swaps = '+repr(natomsswap)+'\n')
    syms=[sym for sym,c in indiv.swaplist]
    if debug: print 'Starting swaplist = '+repr(indiv.swaplist)
    
    #Extend to bulk
    if Optimizer.structure=='Defect':
        indatms=indiv[0].copy()
        brmark=len(indatms)
        indatms.extend(indiv.bulki.copy())
    else:
        indatms=indiv[0].copy()
        brmark=int(len(indatms)/2.0)
    
    #Sanity check
    sanch = [[sym,0] for sym in syms]
    for i in range(len(sanch)):
        nc = len([atm for atm in indatms if atm.symbol==sanch[i][0]])
        nc += [c for sym,c in indiv.swaplist if sym==sanch[i][0]][0]
        sanch[i][1]=nc
    
    # Swap Atoms
    for i in range(natomsswap):
        while True:
            sind=random.choice(indiv.swaplist)
            nsymlist=[sym for sym in syms if sym != sind[0]]
            natlist=[atm for atm in indatms if atm.symbol in nsymlist]
            if sind[1]>0:
                if len(natlist)>0:
                    sym1=sind[0]
                    break
        #Preferentially choose the atoms nearest the defect to swap
        pb = random.random()
        n=0
        while True:
            a1 = random.choice(natlist)
            n += 1
            if pb < 0.5:
                if n < 20:
                    if a1.index < brmark:
                        break
            else:
                if n < 20:
                    if a1.index > brmark:
                        break
            if n > 20:
                break
        nsym=indatms[a1.index].symbol
        indatms[a1.index].symbol=sym1
        Optimizer.output.write('Swapped '+nsym+' atom with '+sym1+'\n')
        for i in range(len(indiv.swaplist)):
            if indiv.swaplist[i][0]==nsym:
                indiv.swaplist[i][1]+=1
            elif indiv.swaplist[i][0]==sym1:
                indiv.swaplist[i][1]-=1
    Optimizer.output.write('New swaplist for individual: '+repr(indiv.swaplist)+'\n')
    if debug: print 'New swaplist = '+repr(indiv.swaplist)
    #Sanity check
    sanchn = [[sym,0] for sym in syms]
    for i in range(len(sanchn)):
        nc = len([atm for atm in indatms if atm.symbol==sanchn[i][0]])
        nc += [c for sym,c in indiv.swaplist if sym==sanchn[i][0]][0]
        sanchn[i][1]=nc
    
    if debug:
        for i in range(len(sanch)):
            print 'INTSWAP: Starting Atom structure '+repr(sanch[i][0])+' = '+repr(sanch[i][1])
            print 'INTSWAP: After Mutation Atom structure '+repr(sanchn[i][0])+' = '+repr(sanchn[i][1])
            
    if Optimizer.structure=='Defect':
        indiv[0]=indatms[0:brmark].copy()
        indiv.bulki=indatms[brmark::].copy()
    else:
        indiv[0]=indatms.copy()
    Optimizer.output.write(repr(indiv[0])+'\n')
    muttype='SC'+repr(natomsswap)
    if indiv.energy==0:
        indiv.history_index=indiv.history_index+'m'+muttype
    else:
        indiv.history_index=repr(indiv.index)+'m'+muttype
    return indiv


##############################################################
# This code is part of the MAterials Simulation Toolkit (MAST)
# 
# Maintainer: Tam Mayeshiba
# Last updated: 2014-05-12 by Zhewen Song
##############################################################
import os, math

from MAST.utility import MASTObj
from MAST.utility import InputOptions
from MAST.utility import MASTError
from MAST.utility import Metadata

ALLOWED_KEYS = {\
                 'templateFile'    : (str, None, 'template file name'),\
                 'inputOptions'    : (InputOptions, None, 'input options parsed using input parser'),\
                 'personalRecipe'  : (str, None, 'personalized recipe file'),\
                 'working_directory' : (str, None, 'Working directory'),
               }

class RecipeTemplateParser(MASTObj):
    """Class for parsing the template recipe file in the input file and
        creating a "personalized" recipe file where any <sys> and <N> from
        the recipe template have been filled in according to information
        from the input file.
        Attributes:
            self.input_options <InputOptions>: Input options from input file.
            self.template_file <str>: Path to the recipe template file
            self.personal_recipe <str>: Path to the personalized recipe.
            self.ingredient_list <list of str>: List of ingredients mentioned
                                                in the recipe template file.
            self.chunks <list of list>: List of chunks
    """
    def __init__(self, **kwargs):
        MASTObj.__init__(self, ALLOWED_KEYS, **kwargs)
        self.input_options   = self.keywords['inputOptions']
        self.template_file   = self.keywords['templateFile']
        self.personal_recipe = self.keywords['personalRecipe']
        self.ingredient_list = list()

        self.metafile = Metadata(metafile='%s/metadata.txt' % self.keywords['working_directory'])
        self.chunks = list()
    def parse(self):
        """ Parses the template recipe file and creates
            the personalized recipe file
        """
        if self.template_file is None:
            raise MASTError(self.__class__.__name__, "Template file not provided!")
        
        self.template_file = os.path.join(os.getenv("MAST_RECIPE_PATH"),self.template_file)


        if not os.path.exists(self.template_file):
            raise MASTError(self.__class__.__name__, "Template file not found!")

        if self.input_options is None:
            raise MASTError(self.__class__.__name__, "Input Options not provided!")

        if self.personal_recipe is None:
            raise MASTError(self.__class__.__name__, "Personal recipe file not provided!")
        
        #fetch required paramaters
        f_ptr           = open(self.template_file, "r")
        o_ptr           = open(self.personal_recipe, "w")
        system_name     = self.input_options.get_item("mast", "system_name", "sys")
        n_defects       = self.input_options.get_item("defects", "num_defects", 0)
        d_defects       = self.input_options.get_item("defects","defects")
        n_images        = self.input_options.get_item("neb", "images", 0)
        d_neblines      = self.input_options.get_item("neb", "neblines", {})
        recipe_name     = None

#        print system_name, self.input_options.get_item('mast', 'system_name')
        chunkcount=0
        mychunk=list()
        modchunk=False
        for line in f_ptr.readlines():
            if '\t' in line:
                raise MASTError("parsers/recipetemplateparser","The tab character exists in recipe template %s. Please convert all tabs to the appropriate number of groups of four spaces." % self.template_file)
            if '{begin}' in line:
                self.chunks.append(list(mychunk))
                mychunk=list()
            elif '{end}' in line:
                pass
            else:
                mychunk.append(line)
        if len(mychunk) > 0:
            self.chunks.append(list(mychunk))
        #for chunk in self.chunks:
        #    print chunk
        expandedlist=list()
        for chunk in self.chunks:
            expanded=self.parse_chunk(chunk)
            expandedlist.extend(expanded)
        o_ptr.writelines(expandedlist)
        f_ptr.close()
        o_ptr.close()
        return recipe_name
        #self.chunks[chunkcount]=dict()
        #self.chunks[chunkcount]['modify'] = modchunk
        #    #line             = line.strip()
        #    #line             = line.lower()
        #    processing_lines = []
        #    #shortcut straight copy + 6
        #    processing_lines.append(line)
        #    output_str = "\n".join(processing_lines)
        #    o_ptr.write("%s\n" % output_str)

        #f_ptr.close()
        #o_ptr.close()
#        print 'in RecipeParser.parse():', list(set(self.ingredient_list))
        #return recipe_name
    def parse_chunk(self, chunk):
        """Parse a chunk of lines.
            Args:
                chunk <list>: List of lines
            Returns:
                expandedchunk <list>: List of lines
        """
        origchunk = list(chunk)
        expandedchunk = list()
        needsdefects=0
        needscharges=0
        needsphonons=0
        needsnebs=0
        needsscaling=0
        for line in chunk:
            if "<N>" in line:
                needsdefects=1
            if "<B-E>" in line:
                needsnebs=1
            if "<P>" in line:
                needsphonons=1
            if "<Q>" in line:
                needscharges=1
            if "<B>" in line:
                needsnebs=1
            if "<E>" in line:
                needsnebs=1
            if "<S>" in line:
                needsscaling=1
        d_scaling       = self.input_options.get_item("structure","scaling")
        d_defects       = self.input_options.get_item("defects","defects")
        d_nebs          = self.input_options.get_item("neb","nebs")
        if needsscaling == 1:
            scalingsize = d_scaling.keys()
            scalingsize.sort()
        else: scalingsize = '1x1x1'

        for size in scalingsize:
            if needsdefects == 1:
                mydefects=d_defects.keys()
                mydefects.sort()
                for defectname in mydefects:
                    for charge in d_defects[defectname]['charge']:
                        if charge < 0:
                            mycharge = 'q=n' + str(int(math.fabs(charge)))
                        else:
                            mycharge = 'q=p' + str(int(charge))
                        if needsphonons == 1:
                            if len(d_defects[defectname]['phonon'].keys()) > 0:
                                phononkeys = d_defects[defectname]['phonon'].keys()
                                phononkeys.sort()
                                for phonon in phononkeys:
                                    for line in origchunk:
                                        newline = line.replace("<N>", defectname)
                                        if needscharges == 1:
                                            newline = newline.replace("<Q>", mycharge)
                                        if needsscaling == 1:
                                            newline = newline.replace("<S>",size)
                                        newline = newline.replace("<P>", phonon)
                                        expandedchunk.append(newline)
                        else:
                            for line in origchunk:
                                newline = line.replace("<N>", defectname)
                                if needscharges == 1:
                                    newline = newline.replace("<Q>", mycharge)
                                if needsscaling == 1:
                                    newline = newline.replace("<S>",size)
                                expandedchunk.append(newline)
            elif needsnebs == 1:
                nebkeys = d_nebs.keys()
                nebkeys.sort()
                for neblabel in nebkeys:
                    defbegin = neblabel.split('-')[0]
                    defend = neblabel.split('-')[1]
                    chargebegin = d_defects[defbegin]['charge']
                    chargeend = d_defects[defend]['charge']
                    chargeboth = set(chargebegin) & set(chargeend)
                    for charge in chargeboth:
                        if charge < 0:
                            mycharge = 'q=n' + str(int(math.fabs(charge)))
                        else:
                            mycharge = 'q=p' + str(int(charge))
                        if needsphonons == 1:
                            if len(d_nebs[neblabel]['phonon'].keys()) > 0:
                                phononkeys = d_nebs[neblabel]['phonon'].keys()
                                phononkeys.sort()
                                for phonon in phononkeys:
                                    for line in origchunk:
                                        newline = line.replace("<B>", defbegin)
                                        newline = newline.replace("<E>", defend)
                                        newline = newline.replace("<B-E>", neblabel)
                                        if needscharges == 1:
                                            newline = newline.replace("<Q>", mycharge)
                                        if needsscaling == 1:
                                            newline = newline.replace("<S>",size)
                                        newline = newline.replace("<P>", phonon)
                                        expandedchunk.append(newline)
                        else:
                            for line in origchunk:
                                newline = line.replace("<B>", defbegin)
                                newline = newline.replace("<E>", defend)
                                newline = newline.replace("<B-E>", neblabel)
                                if needscharges == 1:
                                    newline = newline.replace("<Q>", mycharge)
                                if needsscaling == 1:
                                    newline = newline.replace("<S>",size)                            
                                expandedchunk.append(newline)
            elif needsscaling==1:
                for line in origchunk:
                    newline = line.replace("<S>",size)
                    expandedchunk.append(newline)
            else: expandedchunk = list(origchunk)
        return expandedchunk
        #origchunk = list(expandedchunk)
        #expandedchunk=list()
        #for defectname in self.d_defects:
        #    for line in origchunk:
        #        newline = line.replace("<N>", defectname)
        #        expandedchunk.append(line)


    def old_parsing(self):
        raise MASTError(self.__class__.__name__, "This function is obsolete.") 
        for line in linestr:
            #validate the input line
            if not line or line.startswith('#'):
                continue

            #collect recipe name
            line2 = line.split()
            if (line2[0].lower() == 'recipe'):
                recipe_name = line2[1] 

            #collect ingredents
            line2 = line.split()
            if (line2[0].lower() == 'ingredient'):
                self.ingredient_list.append(line2[2])


            #replace line params<N>
            #step 1 : replace <sys> with system_name
            #step 2 : replace <n-n> with appropriate hop combinations
            #step 3 : replace <img-n> with appropriate image numbers
            #step 4 : replace <n> with appropriate defect numbers
            processing_lines.append(line)
            #step 1
            processing_lines = self.process_system_name(processing_lines, system_name)
            #step 4
            processing_lines = self.process_defects(processing_lines, n_defects, d_defects)
            #step 2
            processing_lines = self.process_hop_combinations(processing_lines, d_neblines)
            #step 3
            processing_lines = self.process_images(processing_lines, n_images)

            self.make_metadata_entries(processing_lines)

            self.process_phononlines(processing_lines)
            #dump the processed lines to file
            output_str = "\n".join(processing_lines)
            o_ptr.write("%s\n" % output_str)

        f_ptr.close()
        o_ptr.close()
#        print 'in RecipeParser.parse():', list(set(self.ingredient_list))
        return recipe_name

    def process_system_name(self, processing_lines, system_name):
        """replace <sys> with the system name from the input options
        """
        raise MASTError(self.__class__.__name__, "This function is obsolete.") 
        for index in xrange(len(processing_lines)):
            processing_lines[index] = processing_lines[index].replace('<sys>', system_name)
        return processing_lines

    def process_hop_combinations(self, processing_lines, d_neblines):
        """replace <n-n> with neb labels which are keys of the
           neblines dict of the input options.

            Args:
                processing_lines <list of str>: recipe lines to process.
                d_neblines <dict of str>: dictionary of NEB lines.
        """
        raise MASTError(self.__class__.__name__, "This function is obsolete.") 
        new_lines = []
        eval_lines = []
        if not d_neblines:
            return processing_lines
        line=""
        for line in processing_lines:
            if "<n-n>" in line:
                for neblabel in d_neblines.keys():
                    n_line = line.replace('<n-n>', neblabel)
                    eval_lines.append(n_line)
                    if 'ingredient' in n_line and not '<' in n_line: 
                        keyword = n_line.split()[1]
                        data = 'neblabel: %s' % neblabel
                        self.metafile.write_data(keyword, data)
            else:
                eval_lines.append(line)
        line=""
        for line in eval_lines:
            #print "TTM DEBUG line: ", line
            if not 'neb' in line.split('_'):
                #print "TTM DEBUG line safe"
                new_lines.append(line)
            else:
                evalsplit = line.split('child')
                if len(evalsplit) == 1:
                    new_lines.append(line)
                else:
                    childsplit=evalsplit[1].split('_')
                    parsplit=evalsplit[0].split('_')
                    okay=1
                    for neblabel in d_neblines.keys():
                        if okay == 1 and (neblabel in childsplit):
                            parlabels = neblabel.split('-')
                            #print "TTM DEBUG: parlabels: ",parlabels
                            #print "TTM DEBUG: parsplit: ",parsplit
                            if not parlabels[0] in parsplit and not (parlabels[1] in parsplit):
                                if not neblabel in parsplit: #image static
                                    okay=0
                    if okay == 0:
                        pass
                    else:
                        new_lines.append(line)
        return new_lines

    def process_images(self, processing_lines, n_images):
        """replace <img-n> with the equivalent number of 
           lines based on the number of images found in the
           input_options
        """
        raise MASTError(self.__class__.__name__, "This function is obsolete.") 
        new_lines = []
        if not n_images:
            return processing_lines
        for line in processing_lines:
            if '<img-n>' in line:
                for index in xrange(n_images):
                    n_line = line.replace('<img-n>', str(index+1).zfill(2))
                    if 'ingredient' in n_line and not '<' in n_line: 
                        keyword = n_line.split()[1]
                        data = 'name: %s' % keyword
                        self.metafile.write_data(keyword, data)
                    new_lines.append(n_line)
            else:
                 new_lines.append(line)
        return new_lines

    def process_defects(self, processing_lines, n_defects, d_defects):
        """replace <N> with the equivalent number of lines
           based on the number of defects given in the
           input options

           Args:
            processing_lines <list of str>: recipe lines to proceess
            n_defects <int>: number of defected systems
            d_defects <dict>: dictionary of defects, including labels and 
                                positions.
        """
        raise MASTError(self.__class__.__name__, "This function is obsolete.") 
        #import inspect
        #print 'GRJ DEBUG: %s.%s' % (self.__class__.__name__, inspect.stack()[0][3])
        #print d_defects

        #print 'GRJ DEBUG: parse_defects() working_directory =', self.keywords['working_directory']
        new_lines = list()

        if not n_defects:
            return processing_lines

        for line in processing_lines:
            #print 'GRJ DEBUG: line =', line
            if ('<n>' in line) or ('<q>' in line):
                for defect_key in d_defects.keys():
                    #print 'GRJ DEBUG: defect_key =', defect_key
                    #defect_label = defect_key.split('_')[1] #defect_1, etc.
                    def_line = line.replace("<n>", defect_key)

                    charge_list = d_defects[defect_key]['charge']
                    #print 'GRJ DEBUG: charge_list =', charge_list
                    #print 'GRJ DEBUG: def_line before charges:', def_line
                    for charge in charge_list:
                        if (charge < 0):
                            clabel = 'q=n' + str(abs(charge))
                        else:
                            clabel = 'q=p' + str(charge)
                        #print 'GRJ DEBUG: clabel =', clabel
                        new_def_line = def_line.replace('<q>', clabel)
                        new_lines.append(new_def_line)
                        #new_lines.append(def_line.replace('<q>', clabel))

                        if 'ingredient' in def_line:
                            keyword = new_def_line.split()[1]
                            data = 'defect_label: %s, charge: %i' % (defect_key, charge)
                            #print 'GRJ DEBUG: def_line, keyword, charge', new_def_line, keyword, charge
                            self.metafile.write_data(keyword, data)
            else:
                new_lines.append(line)

        return new_lines

    def process_phononlines(self, processing_lines):
        """add phonon information to the metadata. Does not change line info.
        """
        raise MASTError(self.__class__.__name__, "This function is obsolete.") 
        for line in processing_lines:
            if 'ingredient' in line and 'phonon_' in line:
                nameval = line.split()[1]
                [dataline,dataval]=self.metafile.search_data(nameval)
                okay=0
                if not (dataval == None):
                    datapcs = dataval.split(',')
                    for datapc in datapcs:
                        dlabel = datapc.split(":")[0].strip()
                        dval = datapc.split(":")[1].strip()
                        if dlabel == 'neblabel' or dlabel == 'defect_label':
                            data = 'phononlabel: %s' % dval
                            self.metafile.write_data(nameval, data)
                            okay=1
                            break
                if okay==0:
                    if 'perfect' in line:
                        data = 'phononlabel: perfect'
                        self.metafile.write_data(nameval, data)
                        okay=1
                    else:
                        data = 'phononlabel: %s' % nameval
                        self.metafile.write_data(nameval, data)
                        okay=1
        return

    def make_metadata_entries(self, processing_lines):
        """Add metadata entry for all ingredients. 
            Does not change line information.
        """
        raise MASTError(self.__class__.__name__, "This function is obsolete.") 
        for line in processing_lines:
            if 'ingredient' in line:
                nameval = line.split()[1]
                data = 'name: %s' % nameval
                self.metafile.write_data(nameval, data)
        return 
    def get_unique_ingredients(self):
        """fetches the ingredients names"""
        raise MASTError(self.__class__.__name__, "This function is obsolete.") 
        return list(set(self.ingredient_list))



import os
import copy
import random
import time
import logging
from MAST.structopt.inp_out.loggerUtils import initialize_logger
from MAST.structopt.tools.check_atomlist_concentration import check_atomlist_concentration
import pdb
try:
    from mpi4py import MPI
except:
    pass

def read_parameter_input(input, logger):
    """Function to convert input string, file, or dictionary to a dictionary to contain 
        the parameters for use by the optimizer class.
        input:
            input : Can be dictionary, string, or filename
            logfile : Name of logfile to write to. Default is None
        output:
            parameters : dictionary for defining parameters for optimizer with defaults
    """
    parameters = None
    if isinstance(input,dict):
        # If supplied input is already a dictionary set parameters equal to that dictionary
        parameters=input
    else:
        #Check to see if input is a filename
        if os.path.exists(input):
            try:
                #Check if input file is already formatted as a python dictionary
                parameters = eval('{%s}' % open(input).read())
            except:
                f = open(input,'r')
                lines = f.readlines()
                f.close()
        else:
            #lines = input
            #Split lines in input string
            lines = input.rstrip().split('\n')
        if parameters==None:
            parameters = {}
            for one in lines:
                if one != '\n':
                    p, v = one.strip().split('=')
                    try:
                        #Convert numbers to floats
                        parameters[p.strip()] = float(v.strip())
                    except:
                        try:
                            #Convert list, tuple, and booleans
                            if p.strip() != 'calc_method':
                                parameters[p.strip()] = eval(v.strip())
                            else:
                                parameters[p.strip()] = v.strip()
                        except:
                            try:
                                #Leave remaining as strings
                                parameters[p.strip()] = v.strip()
                            except:
                                print 'Trouble with input line: ', one
    if 'parallel' not in parameters:
        parameters['parallel'] = False
    else:
        parameters['parallel'] = bool(parameters['parallel'])
    if 'filename' not in parameters:
        parameters['filename']='Output'
    else:
        parameters['filename']=str(parameters['filename'])
    try:
        rank = MPI.COMM_WORLD.Get_rank()
    except:
        rank = 0
    if logger:
        if 'loggername' not in parameters:
            parameters['loggername']= '{0}-rank{1}-{2}.log'.format(parameters['filename'],rank,time.strftime("%Y_%m%d_%H%M%S"))
            logger = initialize_logger(parameters['loggername'])
        else:
            parameters['loggername'] = str(parameters['loggername'])
            logger = initialize_logger(parameters['loggername'])
            #logging.getLogger(parameters['loggername'])
    else:
        parameters['loggername'] = None
        logger = dummy_logger_no_write()
    if ('atomlist' not in parameters):
        #Stop program if atom list parameter not in input
        logger.critical("Input file/string/dictionary must include an atomlist defined as 'atomlist':[('Xx',Concentration,Mass,Chemical Potential)]")
        logger.critical("Current parameters include:\n" + repr(parameters))
        raise RuntimeError("Input file/string/dictionary must include an atomlist defined as 'atomlist':[('Xx',Concentration,Mass,Chemical Potential)]")
    else:
        if isinstance(parameters['atomlist'],str):
            parameters['atomlist'] = eval(parameters['atomlist'].strip())
        if not isinstance(parameters['atomlist'],list):
            logger.critical('Something is wrong with atomlist parameter: {0}'.format(parameters['atomlist']))
            raise RuntimeError("Input file/string/dictionary must include an atomlist defined as 'atomlist':[('Xx',Concentration,Mass,Chemical Potential)]: {0}".format(parameters['atomlist']))
    if 'structure' not in parameters:
        #Stop program if structure parameter not in input
        logger.critical("Input file/dictionary must include a structure for the simulation as 'structure':'Cluster/Crystal/Defect'")
        logger.debug("Current parameters include:\n"+repr(parameters))
        raise RuntimeError("Input file/dictionary must include a structure for the simulation as 'structure':'Cluster/Crystal/Defect'")
    for one in parameters['atomlist']:
        if len(one) != 4:
            #Stop program if atom list parameter not properly formatted
            logger.critical('Format of atom list not correct. Must be [(symbol,concentration,mass,potential)]')
            logger.debug('Issue in section : {0}'.format(one))
            logger.debug('Current atomlist is formatted as : {0}'.format(parameters['atomlist']))
            raise RuntimeError('Format of atom list not correct. Must be [(symbol,concentration,mass,potential)]')
    if 'natoms' not in parameters:
        parameters['natoms'] = int(sum([abs(c) for ind,c,m,u in parameters['atomlist']]))
        if rank==0:
            logger.warning('Number of atoms in simulation not set')
            logger.warning('Assuming natoms = {0}'.format(parameters['natoms']))
    else:
        parameters['natoms'] = int(parameters['natoms'])
    parameters['atomlist'] = check_atomlist_concentration(parameters['atomlist'],parameters['natoms'],parameters['loggername'])
    if 'optimizer_type' not in parameters:
        parameters['optimizer_type'] = 'Random'
        if rank==0:
            logger.info('optimizer_type not set.  Default values set to Random.')
    
    #Set parameter defaults based on optimizer structure
    if parameters['optimizer_type'] == 'GA':
        nindiv = 10
        genealogy = True
        nbests = 100
        algtype = 'lambda+mu'
        cxpb = 0.8
        mutpb = 0.15
        natselectscheme = 'tournament'
    else:
        nindiv = 1
        genealogy = False
        nbests = 100
        algtype = parameters['optimizer_type']
        cxpb = 0.0
        mutpb = 1.0
    if parameters['optimizer_type'] == 'SA':
        natselectscheme = 'metropolis'
        predator = 'adapting'
    elif parameters['optimizer_type'] == 'BH':
        natselectscheme = 'metropolis'
    else:
        natselectscheme = 'best'
    if 'nindiv' not in parameters:
        parameters['nindiv'] = nindiv
        if rank==0:
            logger.info('Setting number of individuals in population (nindiv) = {0}'.format(parameters['nindiv']))
    else:
        parameters['nindiv'] = int(parameters['nindiv'])
    
    #Parameters for output
    if 'genealogy' not in parameters:
        parameters['genealogy'] = genealogy
        if rank==0:
            logger.info('Setting genealogy = {0}'.format(parameters['genealogy']))
    else:
        parameters['genealogy'] = bool(parameters['genealogy'])
    if 'output_format' not in parameters:
        parameters['output_format'] = 'fitness'
        if rank==0:
            logger.info('Setting output format = {0}'.format(parameters['output_format']))
    else:
        parameters['output_format'] = str(parameters['output_format']).lower()
    if 'allenergyfile' not in parameters:
        parameters['allenergyfile'] = False
        if rank==0:
            logger.info('Setting allenergyfile = {0}'.format(parameters['allenergyfile']))
    else:
        parameters['allenergyfile'] = bool(parameters['allenergyfile'])
    if 'best_inds_list' not in parameters:
        parameters['best_inds_list'] = True
        if rank==0:
            logger.info('Setting best_inds_list = {0}'.format(parameters['best_inds_list']))
    else:
        parameters['best_inds_list'] = bool(parameters['best_inds_list'])
    if 'number_of_bests' not in parameters:
        parameters['number_of_bests'] = nbests
        if rank==0:
            logger.info('Setting number_of_bests = {0}'.format(parameters['number_of_bests']))
    else:
        parameters['number_of_bests'] = int(parameters['number_of_bests'])
    if 'indiv_defect_write' not in parameters:
        parameters['indiv_defect_write'] = False
        if rank==0:
            logger.info('Setting indiv_defect_write = {0}'.format(parameters['indiv_defect_write']))
    else:
        parameters['indiv_defect_write'] = bool(parameters['indiv_defect_write'])
    if 'vacancy_output' not in parameters:
        parameters['vacancy_output'] = False
        if rank==0:
            logger.info('Setting vacancy_output = {0}'.format(parameters['vacancy_output']))
    else:
        parameters['vacancy_output'] = bool(parameters['vacancy_output'])
    if 'restart_optimizer' not in parameters:
        parameters['restart_optimizer'] = False
    else:
        parameters['restart_optimizer'] = bool(parameters['restart_optimizer'])
    #Parameters for post-processing
    if 'lattice_concentration' not in parameters:
        parameters['lattice_concentration'] = False
        if rank==0:
            logger.info('Setting lattice_concentration = {0}'.format(parameters['lattice_concentration']))
    else:
        parameters['lattice_concentration'] = bool(parameters['lattice_concentration'])
    if 'postprocessing' not in parameters:
        parameters['postprocessing'] = False
        if rank==0:
            logger.info('Setting postprocessing = {0}'.format(parameters['postprocessing']))
    else:
        parameters['postprocessing'] = bool(parameters['postprocessing'])
    if 'genealogytree' not in parameters:
        parameters['genealogytree'] = False
        if rank==0:
            logger.info('Setting genealogytree = {0}'.format(parameters['genealogytree']))
    else:
        parameters['genealogytree'] = bool(parameters['genealogytree'])
    
    #Parameters for general algorithm
    if 'seed' not in parameters:
        parameters['seed']=random.randint(0,10)
        if rank==0:
            logger.info('Setting Random number seed (seed) to {0}'.format(parameters['seed']))
    else:
        parameters['seed'] = int(parameters['seed'])
    if 'forcing' not in parameters:
        parameters['forcing'] = 'Concentration'
        if rank==0:
            logger.info('Setting forcing = {0}'.format(parameters['forcing']))
            logger.info('Assuming forcing concentration control')
    else:
        parameters['forcing'] = str(parameters['forcing'])
    if 'debug' not in parameters:
        parameters['debug'] = ['None']
        if rank==0:
            logger.info('Setting debug = {0}'.format(parameters['debug']))
    else:
        parameters['debug'] = list(parameters['debug'])
        if 'None' not in parameters['debug']: 
            print '***** DEBUGGING RUN *****'
    if 'algorithm_type' not in parameters:
        parameters['algorithm_type'] = algtype
        if rank==0:
            logger.info('Setting algorithm type = {0}'.format(parameters['algorithm_type']))
    else:
        parameters['algorithm_type'] = str(parameters['algorithm_type'])
    if 'migration_intervals' not in parameters:
        parameters['migration_intervals'] = 5
        if rank==0:
            logger.info('Setting migration_intervals = '.format(parameters['migration_intervals']))
    else:
        parameters['migration_intervals'] = int(parameters['migration_intervals'])
    if 'migration_percent' not in parameters:
        parameters['migration_percent'] = 0.05
        if rank==0:
            logger.info('Setting migration_percent = {0}'.format(parameters['migration_percent']))
    else:
        parameters['migration_percent'] = float(parameters['migration_percent'])
    if 'fingerprinting' not in parameters:
        parameters['fingerprinting'] = False
        if rank==0:
            logger.info('Setting fingerprinting = {0}'.format(parameters['fingerprinting']))
    else:
        parameters['fingerprinting'] = bool(parameters['fingerprinting'])
    if 'fpbin' not in parameters:
        parameters['fpbin'] = 0.25
        if rank==0:
            logger.info('Setting fingerprint bin to {0}'.format(parameters['fpbin']))
    else:
        parameters['fpbin'] = float(parameters['fpbin'])
    if 'fpcutoff' not in parameters:
        parameters['fpcutoff'] = 15.0
        if rank==0:
            logger.info('Setting fingerprint cutoff distance to {0}'.format(parameters['fpcutoff']))
    else:
        parameters['fpcutoff'] = float(parameters['fpcutoff'])
    parameters['bulkfp'] = None
    if 'fixed_region' not in parameters:
        parameters['fixed_region'] = False
        if rank==0:
            logger.info('Setting fixed_region = {0}'.format(parameters['fixed_region']))
    else:
        parameters['fixed_region'] = bool(parameters['fixed_region'])
    if 'rattle_atoms' not in parameters:
        parameters['rattle_atoms'] = False
        if rank==0:
            logger.info('Setting rattle_atoms = {0}'.format(parameters['rattle_atoms']))
    else:
        parameters['rattle_atoms'] = bool(parameters['rattle_atoms'])
    if 'constrain_position' not in parameters:
        parameters['constrain_position'] = False
        if rank==0:
            logger.info('Setting constrain_position = {0}'.format(parameters['constrain_position']))
    else:
        parameters['constrain_position'] = bool(parameters['constrain_position'])
    if 'restart' not in parameters:
        parameters['restart'] = False
        if rank==0:
            logger.info('Setting restart = {0}'.format(parameters['restart']))
    else:
        parameters['restart'] = bool(parameters['restart'])
    if 'restart_ints' not in parameters:
        parameters['restart_ints'] = 0
        if rank==0:
            if parameters['restart']:
                logger.info('Setting restart_ints = {0}'.format(parameters['restart_ints']))
    else:
        parameters['restart_ints'] = int(parameters['restart_ints'])
    
    # Parameters to generate the population and individual
    if 'r_ab' not in parameters:
        parameters['r_ab'] = 2.5
        if rank==0:
            logger.info('Setting r_ab = {0}'.format(parameters['r_ab']))
    else:
        parameters['r_ab'] = float(parameters['r_ab'])
    if 'size' not in parameters:
        parameters['size'] = parameters['natoms']**0.33333*parameters['r_ab']
        if rank==0:
            logger.info('Setting size to (natoms)^(1/3)*r_ab = {0}'.format(parameters['size']))
    else:
        parameters['size'] = float(parameters['size'])
    if 'generate_flag' not in parameters:
        parameters['generate_flag']='box'
        if rank==0:
            logger.info('Setting default generation scheme = {0}'.format(parameters['generate_flag']))
    else:
        parameters['generate_flag']=str(parameters['generate_flag']).lower()
    parameters['solidbulk'] = None
    if 'sf' not in parameters:
        parameters['sf'] = 1.75
        if rank==0:
            if parameters['structure'] == 'Defect':
                logger.info('Setting size factor for Defect (sf) = {0}'.format(parameters['sf']))
    else:
        parameters['sf'] = float(parameters['sf'])
    if 'supercell' not in parameters:
        parameters['supercell'] = (1,1,1)
        if rank==0:
            if parameters['structure'] == 'Defect':
                logger.info('Setting supercell for Defect (supercell) = {0}'.format(parameters['supercell']))
    else:
        parameters['supercell'] = tuple(parameters['supercell'])
    if 'solidfile' not in parameters:
        if parameters['structure'] == 'Defect':
            logger.critical('Must provide a file for bulk solid if running a defect simulation.')
            raise RuntimeError('Error: Bulk for Defect not specified. Enter name of file for bulk structure as solidfile parameter')
        else:
            parameters['solidfile'] = None
    else:
        parameters['solidfile'] = str(parameters['solidfile'])
    if 'solidcell' not in parameters:
        if rank==0:
            if parameters['structure'] == 'Defect':
                logger.warning('Warning cell size for Bulk Solid not specified assuming distance between 1st and 3rd atom')
        parameters['solidcell'] = None
    else:
        try:
            parameters['solidcell'] = eval('numpy.'+parameters['solidcell'])
        except:
            try:
                parameters['solidcell'] = list(parameters['solidcell'])
            except:
                if rank==0:
                    if parameters['structure'] == 'Defect':
                        logger.warning('Warning cell size for Bulk Solid not recognized assuming distance between 1st and 3rd atom')
                        logger.debug('solidcell input as: {0}'.format(parameters['solidcell']))
                parameters['solidcell'] = None 
    if 'evalsolid' not in parameters:
        parameters['evalsolid'] = False
        if rank==0:
            if parameters['structure'] == 'Defect':
                logger.info('Not evaluating Solid')
    else:
        parameters['evalsolid'] = bool(parameters['evalsolid'])
    if 'finddefects' not in parameters:
        parameters['finddefects'] = True
        if rank==0:
            if parameters['structure'] == 'Defect':
                logger.info('Setting finddefects = {0}'.format(parameters['finddefects']))
    else:
        parameters['finddefects'] = bool(parameters['finddefects'])
    if 'trackvacs' not in parameters:
        parameters['trackvacs'] = False
        if rank==0:
            if parameters['structure'] == 'Defect':
                logger.info('Setting trackvacs = {0}'.format(parameters['trackvacs']))
    else:
        parameters['trackvacs'] = bool(parameters['trackvacs'])
    if 'trackswaps' not in parameters:
        parameters['trackswaps'] = False
        if rank==0:
            if parameters['structure'] == 'Defect':
                logger.info('Setting trackswaps = {0}'.format(parameters['trackswaps']))
    else:
        parameters['trackswaps'] = bool(parameters['trackswaps'])
    if 'random_loc_start' not in parameters:
        parameters['random_loc_start'] = False
        if rank==0:
            if parameters['structure'] == 'Defect':
                logger.info('Setting random_loc_start = {0}'.format(parameters['random_loc_start']))
    else:
        parameters['random_loc_start'] = bool(parameters['random_loc_start'])
    if 'random_vac_start' not in parameters:
        parameters['random_vac_start'] = False
        if rank==0:
            if parameters['structure'] == 'Defect':
                logger.info('Setting random_vac_start = {0}'.format(parameters['random_vac_start']))
    else:
        parameters['random_vac_start'] = bool(parameters['random_vac_start'])
    if 'purebulkenpa' not in parameters:
        if parameters['structure'] == 'Defect':
            parameters['purebulkenpa'] = None
        else:
            parameters['purebulkenpa'] = 0
    if 'natomsbulk' not in parameters:
        if parameters['structure'] == 'Defect':
            parameters['natomsbulk'] = None
        else:
            parameters['natomsbulk']=0
    if 'surfacefile' not in parameters:
        if parameters['structure'] == 'Surface':
            logger.critical('Must provide a file for bulk solid if running a defect simulation.')
            raise RuntimeError('Error: Bulk for Defect not specified. Enter name of file for bulk structure as solidfile parameter')
        else:
            parameters['surfacefile'] = None
    if 'surfacecell' not in parameters:
        parameters['surfacecell'] = None
    if 'surftopthick' not in parameters:
        parameters['surftopthick'] = 0        
    if 'cell_shape_options' not in parameters:
        parameters['cell_shape_options'] = ['cubic', 'hexagonal', 'triclinic', \
        'monoclinic', 'orthorhombic', 'tetragonal']
    else:
        parameters['cell_shape_options'] = list(parameters['cell_shape_options'])
        if parameters['cell_shape_options'] == 'all':
            parameters['cell_shape_options']=['cubic', 'hexagonal', 'triclinic', \
            'monoclinic', 'orthorhombic', 'tetragonal']
    if rank==0:
        if parameters['structure'] == 'Crystal':
            logger.info('Assuming following cell shape options: {0}'.format(parameters['cell_shape_options']))
    if 'alloy' not in parameters:
        parameters['alloy'] = True
        if rank==0:
            logger.info('Setting alloy = {0}'.format(parameters['alloy']))
    else:
        parameters['alloy'] = bool(parameters['alloy'])
    
    #Parameters to Evaluate an Individual
    if 'calc_method' not in parameters:
        if 'vaspcalc' in parameters:
            parameters['calc_method'] = 'VASP'
            if rank==0:
                logger.info('Setting calculation method to VASP. calc_method = {0}'.format(parameters['calc_method']))
        elif 'pair_style' in parameters:
            parameters['calc_method'] = 'LAMMPS'
            if rank==0:
                logger.info('Setting calculation method to LAMMPS. calc_method = {0}'.format(parameters['calc_method']))
        elif 'lammps_min' in parameters:
            parameters['calc_method'] = 'LAMMPS'
            if rank==0:
                logger.info('Setting calculation method to LAMMPS. calc_method = {0}'.format(parameters['calc_method']))
        else:
            parameters['calc_method'] = 'LennardJones'
            if rank==0:
                logger.info('Calculation method not specified assuming ase LennardJones. calc_method = {0}'.format(parameters['calc_method']))
    else:
        parameters['calc_method'] = str(parameters['calc_method'])
    if 'vaspcalc' not in parameters:
        parameters['vaspcalc'] = "Vasp()"
        if rank==0:
            if parameters['calc_method'] == 'VASP':
                logger.info('Setting vaspcalc = {0}'.format(parameters['vaspcalc']))
    if 'pair_style' not in parameters:
        if rank==0:
            if parameters['calc_method'] == 'LAMMPS':
                logger.info('No pair_style for LAMMPS specified. Assuming Lennard-Jones')
        parameters['pair_style'] = None
    if 'bopcutoff' not in parameters:
        if parameters['pair_style'] == 'bop':
            logger.critical('BOP potential requires a cutoff distance.')
            logger.debug('Parameters = {0}'.format(parameters))
            raise RuntimeError("ERROR:Cutoff distance must be supplied with bop potential! Specify with bopcutoff=N")
        else:
            parameters['bopcutoff'] = 0
    else:
        parameters['bopcutoff'] = float(parameters['bopcutoff'])
    if 'buckcutoff' not in parameters:
        parameters['buckcutoff'] = 1
        if rank==0:
            if parameters['pair_style'] == 'buck':
                logger.warning('Setting buckingham potential cutoff = ' +repr(parameters['buckcutoff']))
    else:
        parameters['buckcutoff'] = int(parameters['buckcutoff'])
    if 'buckparameters' not in parameters:
        parameters['buckparameters'] = ['* * 100.00 1.5 200.0']
        if rank == 0:
            if parameters['pair_style'] == 'buck':
                logger.warning('Setting Buckingham parameters = {0}'.format(parameters['buckparameters']))
    if 'ps_name' not in parameters:
        parameters['ps_name'] = None
        if rank == 0:
            if parameters['pair_style'] == 'other':
                logger.info('Setting ps_name = {0}'.format(parameters['ps_name']))
    if 'pair_coeff' not in parameters:
        if parameters['pair_style'] == 'other':
            logger.critical('Must provide pair_coeff for use with pair_style = other')
            raise RuntimeError("ERROR:Coefficients for structure=other potential not specifed. Use pair_coeff")
        else:
            parameters['pair_coeff'] = None
    if 'ps_other' not in parameters:
        parameters['ps_other'] = None
        if rank==0:
            if parameters['pair_style'] == 'other':
                logger.info('Setting ps_other = {0}'.format(parameters['ps_other']))
    if 'pot_file' not in parameters:
        parameters['pot_file'] = None
        if rank == 0:
            if parameters['calc_method'] == 'LAMMPS':
                logger.warning('No potential file for LAMMPS specified.')
    else:
        parameters['pot_file'] = str(parameters['pot_file'])
    if 'lammps_keep_files' not in parameters:
        parameters['lammps_keep_files'] = False
        if rank == 0:
            if parameters['calc_method'] == 'LAMMPS':
                logger.info('Setting lammps_keep_files = {0}'.format(parameters['lammps_keep_files']))
    else:
        parameters['lammps_keep_files'] = bool(parameters['lammps_keep_files'])
    if 'lammps_thermo_steps' not in parameters:
        parameters['lammps_thermo_steps'] = 1
        if rank == 0:
            if parameters['calc_method'] == 'LAMMPS':
                logger.info('Setting lammps_thermo_steps = {0}'.format(parameters['lammps_thermo_steps']))
    else:
        parameters['lammps_thermo_steps'] = int(parameters['lammps_thermo_steps'])
    if ('lammps_min' in parameters) and (parameters['lammps_min'] != None):
        parameters['ase_min'] = False
    else:
        parameters['lammps_min'] = None
        if rank ==0:
            if parameters['calc_method'] == 'LAMMPS':
                logger.info('No Local minimization implemented in LAMMPS')
    if 'lammps_min_style' not in parameters:
        if parameters['lammps_min']:
            parameters['lammps_min_style'] = 'cg'
            if rank ==0:
                if parameters['calc_method'] == 'LAMMPS':
                    logger.info('Setting lammps_min_style = {0}'.format('lammps_min_style')+'. Using LAMMPS conjugate gradient local minimizer')
        else:
            parameters['lammps_min_style'] = None
    if 'large_box_size' not in parameters:
        parameters['large_box_size']=500.0
        if rank == 0:
            if parameters['structure']=='Cluster':
                logger.info('Setting large_box_size to {0}'.format(parameters['large_box_size']))
    else:
        parameters['large_box_size']=float(parameters['large_box_size'])
    if 'ase_min' not in parameters:
        if parameters['calc_method']=='LennardJones':
            parameters['ase_min'] = True
        else:
            parameters['ase_min'] = False
        if rank==0:
            logger.info('Setting ase_min = {0}'.format(parameters['ase_min']))
    else:
        parameters['ase_min'] = bool(parameters['ase_min'])
    if 'ase_min_fmax' not in parameters:
        parameters['ase_min_fmax'] = 0.01
        if rank ==0:
            if parameters['ase_min']:
                logger.info('Setting ASE maximum force value (ase_min_fmax) = {0}'.format(parameters['ase_min_fmax']))
    else:
        parameters['Sovlerfmax'] = float(parameters['ase_min_fmax'])
    if 'ase_min_maxsteps' not in parameters:
        parameters['ase_min_maxsteps'] = 2500
        if rank ==0:
            if parameters['ase_min']:
                logger.info('Setting maximum number of steps for BFGS solver (ase_min_maxsteps) = {0}'.format(parameters['ase_min_maxsteps']))
    else:
        parameters['SovlerMxSteps'] = int(parameters['ase_min_maxsteps'])
    
    #Parameters for Crossovers
    if 'cxpb' not in parameters:
        parameters['cxpb'] = cxpb
        if rank == 0:
            logger.info('Setting crossover probability (cxpb) = {0}'.format(parameters['cxpb']))
    else:
        parameters['cxpb'] = float(parameters['cxpb'])
    if 'cx_scheme' not in parameters:
        parameters['cx_scheme'] = 'cxtp'
        if rank == 0:
            logger.info('Assuming two-point crossover.  Setting cx_scheme = {0}'.format(parameters['cx_scheme']))
    else:
        parameters['cx_scheme'] = str(parameters['cx_scheme']).lower()
    if 'selection_scheme' not in parameters:
        parameters['selection_scheme'] = 'tournament2'
        if rank == 0:
            logger.info('Setting selection_scheme = {0}'.format(parameters['selection_scheme']))
    else:
        parameters['selection_scheme'] = str(parameters['selection_scheme']).lower()
    
    #Parameters for Mutations
    if 'mutpb' not in parameters:
        parameters['mutpb'] = mutpb
        if rank == 0:
            logger.info('Setting mutation probability (mutpb) = {0}'.format(parameters['mutpb']))
    else:
        parameters['mutpb'] = float(parameters['mutpb'])
    if 'mutation_options' not in parameters:
        if parameters['structure']=='Cluster':
            parameters['mutation_options']=['lattice_alteration','rotation',\
            'permutation','scale_size']
        elif parameters['structure']=='Crystal':
            parameters['mutation_options']=['lattice_alteration','rotation',\
            'permutation','scale_size', 'cell_shape', 'lammps_box_relax']
        elif parameters['structure']=='Defect':
            parameters['mutation_options']=['lattice_alteration','rotation','permutation']
        if rank == 0:
            logger.info('Setting mutations options = {0}'.format(parameters['mutation_options']))
    else:
        parameters['mutation_options'] = list(parameters['mutation_options'])
        for i in range(len(parameters['mutation_options'])):
            parameters['mutation_options'][i] = parameters['mutation_options'][i].lower()
    BHFlag=False
    for one in parameters['mutation_options']:
        if 'basin_hop' in one:
            BHFlag=True
    if 'bh_steps' not in parameters:
        parameters['bh_steps']=100
        if rank == 0:
            if BHFlag==True:
                logger.warning('Max steps not specified for Basin Hop mutation, setting bh_steps = {0}'.format(parameters['bh_steps']))
    else:
        parameters['bh_steps'] = int(parameters['bh_steps'])
    if 'bh_temp' not in parameters:
        parameters['bh_temp'] = 1000*8.617385692256675e-05
        if rank == 0:
            if BHFlag==True:
                logger.warning('Temperature not set for Basin Hop mutation, setting bh_temp in kT = {0}'.format(parameters['bh_temp']))
    else:
        parameters['bh_temp'] = float(parameters['bh_temp'])
    if 'mutant_add' not in parameters:
        parameters['mutant_add'] = False
        if rank == 0:
            logger.info('Setting mutant_add = {0}'.format(parameters['mutant_add']))
    else:
        parameters['mutant_add'] = bool(parameters['mutant_add'])
    if 'quench_max_temp' not in parameters:
        parameters['quench_max_temp'] = 1000
        if rank == 0:
            if 'quench' in parameters['mutation_options']:
                logger.info('Setting quench_max_temp = {0}'.format(parameters['quench_max_temp']))
    else:
        parameters['quench_max_temp'] = int(parameters['quench_max_temp'])
    if 'quench_min_temp' not in parameters:
        parameters['quench_min_temp'] = 2
        if rank == 0:
            if 'quench' in parameters['mutation_options']:
                logger.info('Seting quench_min_temp = {0}'.format(parameters['quench_min_temp']))
    else:
        parameters['quench_min_temp'] = int(parameters['quench_min_temp'])
    if 'quench_step_size' not in parameters:
        parameters['quench_step_size'] = 0.01
        if rank == 0:
            if 'quench' in parameters['mutation_options']:
                logger.info('Setting quench_step_size = {0}'.format(parameters['quench_step_size']))
    else:
        parameters['quench_step_size'] = float(parameters['quench_step_size'])
    if 'quench_n_steps_1' not in parameters:
        parameters['quench_n_steps_1'] = 10000
        if rank == 0:
            if 'quench' in parameters['mutation_options']:
                logger.info('Setting quench_n_steps_1 = {0}'.format(parameters['quench_n_steps_1']))
    else:
        parameters['quench_n_steps_1'] = int(parameters['quench_n_steps_1'])
    if 'quench_n_steps_2' not in parameters:
        parameters['quench_n_steps_2'] = parameters['quench_n_steps_1']*2
        if rank == 0:
            if 'quench' in parameters['mutation_options']:
                logger.info('Setting quench_n_steps_2 = {0}'.format(parameters['quench_n_steps_2']))
    else:
        parameters['quench_n_steps_2'] = int(parameters['quench_n_steps_2'])
    if 'isolate_mutation' not in parameters:
        parameters['isolate_mutation'] = False
        if rank == 0:
            logger.info('Setting isolate_mutation flag = {0}'.format(parameters['isolate_mutation']))
    else:
        parameters['isolate_mutation'] = bool(parameters['isolate_mutation'])
    
    #Parameters for Selection
    if 'fitness_scheme' not in parameters:
        parameters['fitness_scheme'] = 'totalenfit'
        if rank == 0:
            logger.info('Setting fitness_scheme = {0}'.format(parameters['fitness_scheme']))
    else:
        parameters['fitness_scheme'] = str(parameters['fitness_scheme']).lower()
    if 'energy_cutoff_factor' not in parameters:
        parameters['energy_cutoff_factor'] = 10.0
        if rank ==0:
            logger.info('Setting energy_cutoff_factor = {0}'.format(parameters['energy_cutoff_factor']))
    else:
        parameters['energy_cutoff_factor'] = float(parameters['energy_cutoff_factor'])
    if 'stem_parameters' not in parameters:
        if 'stem' in parameters['fitness_scheme']:
            logger.critical('Must provide stem_parameters for STEM_Cost calculation')
            raise RuntimeError("STEM parameters not specified.  Cannot simulate image files")
        else:
            parameters['stem_parameters'] = {}
    else:
        stemparams = dict(parameters['stem_parameters'])
    if 'stem_keep_files' not in parameters:
        parameters['stem_keep_files'] = True
        if rank == 0:
            if 'stem' in parameters['fitness_scheme']:
                logger.info('Setting stem_keep_files = {0}'.format(parameters['stem_keep_files']))
    else:
        parameters['stem_keep_files'] = parameters['stem_keep_files']
    if 'stem_coeff' not in parameters:
        parameters['stem_coeff'] = None
        if rank == 0:
            if 'stem' in parameters['fitness_scheme']:
                logger.info('Setting stem_coeff with first individual')
    else:
        try:
            parameters['stem_coeff'] = float(parameters['stem_coeff'])
        except:
            if rank == 0:
                if 'stem' in parameters['fitness_scheme']:
                    logger.warning('Trouble reading stem_coeff input. stem_coeff = {0}'.format(parameters['stem_coeff']))
            parameters['stem_coeff'] = None
    if 'stem' in parameters['fitness_scheme']:
        #Initialize function for experimental image
        from MAST.structopt.tools.StemCalc import ConvStem
        logger.info('Initializing ConvStem Calculator')
        parameters['stemcalc'] = ConvStem(parameters=stemparams, tmp_dir='/'+os.getcwd()+'/ConvStemImages/', keep_files=parameters['stem_keep_files'])
    else:
        parameters['stemcalc'] = None
    if 'constrain_swaps' not in parameters:
        if 'IntSwap' in parameters['mutation_options']:
            parameters['swaplist'] = None
            if rank == 0:
                logger.info('Setting swaplist for IntSwap = None')
        elif parameters['fitness_scheme'] == 'chempotswap':
            parameters['swaplist'] = None
            if rank == 0:
                logger.info('Setting swaplist for chempotswap = None')
        else:
            parameters['swaplist'] = False
            parameters['constrain_swaps'] = False
            if rank == 0:
                logger.info('Setting swaplist = False')
    else:
        parameters['swaplist'] = parameters['constrain_swaps']
    if 'natural_selection_scheme' not in parameters:
        parameters['natural_selection_scheme'] = natselectscheme
        if rank ==0:
            logger.info('Setting natural_selection_scheme = {0}'.format(parameters['natural_selection_scheme']))
    else:
        parameters['natural_selection_scheme'] = str(parameters['natural_selection_scheme']).lower()
    if 'tournsize' not in parameters:
        parameters['tournsize'] = 3
        if 'tournament' in parameters['selection_scheme'] or 'tournament' in parameters['natural_selection_scheme']:
            if rank ==0:
                logger.info('Setting Tournament size (tournsize) = {0}'.format(parameters['tournsize']))
    else:
        parameters['tournsize']=int(parameters['tournsize'])
    if 'fusslimit' not in parameters:
        parameters['fusslimit'] = 10.0
        if rank ==0:
            logger.info('Setting FUSS limit (fusslimit) = {0}'.format(parameters['fusslimit']))
    else:
        parameters['fusslimit'] = float(parameters['fusslimit'])
    if 'metropolis_temp' not in parameters:
        parameters['metropolis_temp'] = 30.0
        if rank == 0:
            logger.info('Setting metropolis_temp = {0}'.format(parameters['metropolis_temp']))
    else:
        parameters['metropolis_temp'] = float(parameters['metropolis_temp'])
    if 'mark' not in parameters:
        parameters['mark'] = None
    #Parameters for Convergence
    if 'convergence_scheme' not in parameters:
        parameters['convergence_scheme'] = 'max_gen'
        if rank ==0:
            logger.info('Setting convergence scheme (convergence_scheme) = {0}'.format(parameters['convergence_scheme']))
    else:
        parameters['convergence_scheme'] = str(parameters['convergence_scheme']).lower()
    if 'maxgen' not in parameters:
        parameters['maxgen'] = 5
        if rank ==0:
            logger.info('Setting Max Number of generations (maxgen) = {0}'.format(parameters['maxgen']))
    else:
        parameters['maxgen'] = int(parameters['maxgen'])
    if 'reqrep' not in parameters:
        parameters['reqrep'] = 10
        if rank ==0:
            if 'rep' in parameters['convergence_scheme']:
                logger.info('Setting max number of energy repetitions (reqrep) = {0}'.format(parameters['reqrep']))
    else:
        parameters['reqrep'] = int(parameters['reqrep'])
    if 'tolerance' not in parameters:
        parameters['tolerance'] = 0.001
        if rank ==0:
            if 'rep' in parameters['convergence_scheme']:
                logger.info('Setting energy tolerance (tolerance) = {0}'.format(parameters['tolerance']))
    else:
        parameters['tolerance'] = float(parameters['tolerance'])
    if 'predator' not in parameters:
        parameters['predator'] = 'mutation_dups'
        if rank == 0:
            logger.info('Setting predator = {0}'.format(parameters['predator']))
    else:
        parameters['predator'] = str(parameters['predator']).lower()
    if 'adaptbegin' not in parameters:
        parameters['adaptbegin'] = 0.75
        if rank == 0:
            if parameters['predator'] == 'adapting':
                logger.info('Setting adaptation predator to begin (adaptbegin) at genrep*{0}'.format(parameters['adaptbegin']))
    else:
        parameters['adaptbegin'] = float(parameters['adaptbegin'])
    if 'adaptmultiplier' not in parameters:
        parameters['adaptmultiplier'] = 3.0
        if rank == 0:
            if parameters['predator'] == 'adapting':
                logger.info('Setting adaptation predator multiplier (adaptmultiplier) = {0}'.format(parameters['adaptmulitplier']))
    else:
        parameters['adaptmultiplier'] = float(parameters['adaptmultiplier'])
    if 'demin' not in parameters:
        parameters['demin'] = 0.005
        if rank == 0:
            logger.info('Setting cutoff convergence energy (demin) = {0}'.format(parameters['demin']))
    else:
        parameters['demin'] = float(parameters['demin'])
    
    return parameters

class dummy_logger():
    def __init__(self):
        return
    def critical(self,message):
        print 'CRITICAL: {0}'.format(message)
        return
    def debug(self,message):
        print 'DEBUG: {0}'.format(message)
        return
    def warning(self,message):
        print 'WARNING: {0}'.format(message)
        return
    def info(self,message):
        print 'MESSAGE: {0}'.format(message)
        return

class dummy_logger_no_write():
    def __init__(self):
        return
    def critical(self,message):
        return
    def debug(self,message):
        return
    def warning(self,message):
        return
    def info(self,message):
        return


def check_cell_type(indiv):
    """Basic check of cell type
    *** needs development***
    """
    cell=indiv.get_cell()
    a,b,c=cell
    if b[0] != 0:
        if c[0] != 0:
            cell_type='triclinic'
        else:
            cell_type='hexagonal'
    elif c[0] != 0:
        cell_type='monoclinic'
    elif b[1] != c[2]:
        if b[1] != a[0]:
            cell_type='orthorhombic'
        else:
            cell_type='tetragonal'
    else:
        cell_type='cubic'
    return cell_type




import math

def dirac(x,a=0,sig=None):
    """Simple calculation of the dirac function
    Inputs:
        x = position for function evaluation
        a = dirac move
        sig = dirac spread
    Outputs:
        out = value of dirac function evaluated at x
    """
    if sig==None:
        if x != a:
            out=0
        else:
            out=1
    else:
        out=1/(sig*math.pi**0.5)*math.exp(-(x-a)**2/sig**2)
    return out




##############################################################
# This code is part of the MAterials Simulation Toolkit (MAST)
# 
# Maintainer: Tam Mayeshiba
# Last updated: 2014-05-12 by Zhewen Song
##############################################################
from pymatgen.io.vaspio import Poscar
from pymatgen.io.vaspio import Outcar
from pymatgen.io.vaspio import Potcar
from pymatgen.io.vaspio import Incar
from pymatgen.io.vaspio import Kpoints
from pymatgen.io.vaspio.vasp_output import Vasprun
from pymatgen.io.cifio import CifParser
from pymatgen.core.sites import PeriodicSite
from pymatgen.core.structure import Structure
from MAST.utility import dirutil
from MAST.utility.mastfile import MASTFile
from MAST.utility import MASTError
from MAST.utility.metadata import Metadata
from MAST.ingredients.pmgextend.structure_extensions import StructureExtensions
import os
import shutil
import logging
from MAST.ingredients.checker import VaspChecker
from MAST.ingredients.checker import BaseChecker
class VaspNEBChecker(VaspChecker):
    """VASP checker functions
    """
    def __init__(self, **kwargs):
        allowed_keys = {
            'name' : (str, str(), 'Name of directory'),
            'program_keys': (dict, dict(), 'Dictionary of program keywords'),
            'structure': (Structure, None, 'Pymatgen Structure object')
            }
        BaseChecker.__init__(self, allowed_keys, **kwargs)
        self.metafile = Metadata(metafile='%s/metadata.txt' % self.keywords['name'])

    def get_path_to_write_neb_parent_energy(self, parent):
        """Get the path into which to write the NEB parent
            energy.
            For VASP, the paths are the 00 and 0(N+1) 
            directories, where N is the number of images.
            Args:
                parent (if integer) <int>: 1 for initial endpoint; 2 for
                              final endpoint
                parent (if string) <str>: destination folder specified as 
                              a string, e.g. "03"
        """
        myname = self.keywords['name']
        myimages = self.keywords['program_keys']['mast_neb_settings']['images']
        if parent == 1:
            return os.path.join(myname, "00", "OSZICAR")
        elif parent == 2:
            return os.path.join(myname, str(int(myimages)+1).zfill(2),"OSZICAR")
        elif len(parent) > 1:
            return os.path.join(myname, parent, "OSZICAR")
        else:
            raise MASTError(self.__class__.__name__,"Parent not specified correctly.")

    def set_up_neb_folders(self, image_structures):
        """Set up NEB folders.
            Args:
               image_structures <list of Structure>: List
                   of image structures
        """
        imct=0
        myname = self.keywords['name']
        if 'mast_coordinates' in self.keywords['program_keys'].keys():
            coordstrucs=self.get_coordinates_only_structure_from_input()
            newstrucs=list()
            sidx = 0 #ex. coordstrucs 0, 1, 2 for 3 images
            while sidx < self.keywords['program_keys']['mast_neb_settings']['images']:
                sxtend = StructureExtensions(struc_work1=image_structures[sidx+1].copy(), name=self.keywords['name'])
                newstrucs.append(sxtend.graft_coordinates_onto_structure(coordstrucs[sidx]))
                sidx = sidx + 1
        while imct < len(image_structures):
            imposcar = Poscar(image_structures[imct])
            num_str = str(imct).zfill(2)
            impath = os.path.join(myname, num_str)
            impospath = os.path.join(myname, "POSCAR_" + num_str)
            if 'mast_coordinates' in self.keywords['program_keys'].keys():
                if imct == 0: #skip endpoint
                    pass 
                elif imct == len(image_structures)-1: #skip other endpt
                    pass
                else:
                    imposcar.structure=newstrucs[imct-1].copy()
            dirutil.lock_directory(myname)
            imposcar.write_file(impospath)
            dirutil.unlock_directory(myname)
            try:
                os.makedirs(impath)
            except OSError:
                self.logger.warning("Directory at %s already exists." % impath)
                return None
            dirutil.lock_directory(impath)
            imposcar.write_file(os.path.join(impath, "POSCAR"))
            dirutil.unlock_directory(impath)
            imct = imct + 1
        return
        
    def is_frozen(self):
        """Check if single VASP non-NEB calculation is frozen.
        """
        return BaseChecker.is_frozen(self, "vasprun.xml")
    def is_complete(self):
        """Check if all images in a VASP NEB calculation are complete.
        """
        dirname = self.keywords['name']
        numim = int(self.keywords['program_keys']['mast_neb_settings']['images'])
        imct=1
        while imct <= numim:
            num_str = str(imct).zfill(2)
            impath = os.path.join(dirname, num_str)
            singlechecker=VaspChecker(name=impath,program_keys=self.keywords['program_keys'],structure=self.keywords['structure'])
            if not singlechecker.is_complete():
                return False
            imct = imct + 1
        return True

    def is_ready_to_run(self):
        """Check if single VASP NEB ingredient is 
            ready to run.
        """
        dirname = self.keywords['name']
        notready=0
        if not(os.path.isfile(dirname + "/KPOINTS")):
            notready = notready + 1
        if not(os.path.isfile(dirname + "/POTCAR")):
            notready = notready + 1
        if not(os.path.isfile(dirname + "/INCAR")):
            notready = notready + 1
        else: #INCAR exists. Now check POSCARs.
            subdirs = dirutil.walkdirs(dirname)
            if not (len(subdirs) == self.keywords['program_keys']['mast_neb_settings']['images'] + 2):
                notready = notready + 1 #bad number of subdirs
            subdirs.sort()
            if len(subdirs) == 0: #This is an NEB without folders yet
                notready = notready + 1
            else:
                for subdir in subdirs:
                    if not (os.path.isfile(subdir + "/POSCAR")):
                        notready = notready + 1
                if not os.path.isfile(subdirs[0] + "/OSZICAR"):
                    notready = notready + 1
                if not os.path.isfile(subdirs[-1] + "/OSZICAR"):
                    notready = notready + 1
        if not(os.path.isfile(dirname + "/submit.sh")):
            notready = notready + 1
        if notready > 0:
            return False
        else:
            return True
    
    def _vasp_neb_incar_modify(self):
        """Modify the INCAR to add the IMAGES tag back in.
        """
        name=self.keywords['name']
        images = str(self.keywords['program_keys']['mast_neb_settings']['images'])
        my_incar = MASTFile(self.keywords['name'] + "/INCAR")
        my_incar.data.append("IMAGES=%s\n" % images)
        my_incar.to_file(self.keywords['name'] + "/INCAR")
        return my_incar

    def set_up_program_input(self):
        image_structures = self.keywords['program_keys']['mast_neb_settings']['image_structures']
        self.set_up_neb_folders(image_structures)
        self._vasp_kpoints_setup()
        mypotcar = self._vasp_potcar_setup(Poscar(image_structures[0]))
        self._vasp_incar_setup(mypotcar, Poscar(image_structures[0]))
        self._vasp_neb_incar_modify()
        return

    def get_energy_from_energy_file(self):
        """Get the energy from the energy file.
            For VASP neb, this is the last E0 energy from each
            OSZICAR,
            and this function should be used if vasprun.xml
            is corrupted or not available.
            Args:
                mydir <str>: Directory in which to look.
            Returns:
                <str>: all last E0 energies from OSZICAR files.
        """
        myct=0
        mystr=""
        while myct <= self.keywords['program_keys']['mast_neb_settings']['images']+1:
            fullpath = os.path.join(self.keywords['name'], str(myct).zfill(2), "OSZICAR")
            if not os.path.isfile(fullpath):
                raise MASTError(self.__class__.__name__, "No OSZICAR file at %s" % fullpath)
            myosz = MASTFile(fullpath)
            mye0 = myosz.get_segment_from_last_line_match("E0", "E0=","d E =")
            mye0 = float(mye0)
            mystr = mystr + "%3.3f" % mye0 + ';'
            myct = myct + 1
        mystr=mystr[0:-1] #remove last semicolon
        return mystr

    def is_started(self):
        """See if the ingredient has been started on
            the queue.
        """
        if os.path.isfile(os.path.join(self.keywords['name'],'01','OUTCAR')):
            return True
        else:
            return False



import os
import time
import unittest
from unittest import SkipTest
import filecmp
from filecmp import dircmp
import MAST
import shutil
from MAST.utility import dirutil
from MAST.controllers.mast import MAST as controllerMAST
testname ="mast_test"
testdir = dirutil.get_test_dir(testname)
oldcontrol = os.getenv("MAST_CONTROL")
oldrecipe = os.getenv("MAST_RECIPE_PATH")
oldscratch = os.getenv("MAST_SCRATCH")
print "Old directories:"
print oldcontrol
print oldrecipe
print oldscratch


class TestMAST(unittest.TestCase):
    """Test MAST (mast -i input.inp)
    """
    def setUp(self):
        os.environ['MAST_CONTROL'] = testdir
        os.environ['MAST_RECIPE_PATH'] = testdir
        os.environ['MAST_SCRATCH'] = testdir
        os.chdir(testdir)

    def tearDown(self):
        basicdirs = MAST.utility.dirutil.walkdirs(testdir, 1,1,"*Basic*")
        for basicdir in basicdirs:
            shutil.rmtree(basicdir)
        os.environ['MAST_CONTROL'] = oldcontrol
        os.environ['MAST_RECIPE_PATH'] = oldrecipe
        os.environ['MAST_SCRATCH'] = oldscratch

    def test_check_indep_loop_no_loops(self):
        raise SkipTest
        print "New paths:"
        print os.getenv("MAST_CONTROL")
        print os.getenv("MAST_RECIPE_PATH")
        print os.getenv("MAST_SCRATCH")
        mymast = MAST.mast.MAST(inputfile="basic_test.inp",outputfile="output.inp")
        mymast.check_independent_loops()
    def test_set_up_recipe(self):
        mymast = controllerMAST(inputfile="basic_test.inp",
                                outputfile="output.inp")
        mymast.set_input_options()
        #print "INPUT OPTIONS: ", mymast.input_options
        mymast.set_class_attributes()
        mymast.make_working_directory()
        #print "WORKING DIRECTORY:", mymast.working_directory
        mymast.create_recipe_metadata()
        mymast.copy_input_file()
        mymast.parse_recipe_template()
        mymast.create_recipe_plan()
        mymast.create_archive_files()
        myfolders = MAST.utility.dirutil.walkdirs(testdir,1,1,"*Basic*")
        myfiles = MAST.utility.dirutil.walkfiles(myfolders[-1],1,1)
        subfolders = MAST.utility.dirutil.walkdirs(myfolders[-1],1,1)
        lacksfolders=0
        for folder in ['opt1','opt2']:
            if not os.path.join(myfolders[-1],folder) in subfolders:
                lacksfolders = lacksfolders + 1
        lacksfiles=0
        for file in ['archive_input_options.txt','metadata.txt',
                        'personal_recipe.txt', 'status.txt',
                        'archive_recipe_plan.txt', 'input.inp']:
            if not os.path.join(myfolders[-1],file) in myfiles:
                lacksfiles = lacksfiles + 1
        self.assertEqual(lacksfiles,0)
        self.assertEqual(lacksfolders,0)

